#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests tilted backpropagation algorithm
"""
from __future__ import division, print_function

import numpy as np
import os
from os.path import abspath, basename, dirname, join, split, exists
import platform
import sys
import warnings
import zipfile

# Add parent directory to beginning of path variable
DIR = dirname(abspath(__file__))
sys.path = [split(DIR)[0]] + sys.path

import odtbrain
import odtbrain._Back_2D
import odtbrain._Back_3D
import odtbrain._Back_3D_tilted
import odtbrain._br

from common_methods import create_test_sino_2d, create_test_sino_3d, create_test_sino_3d_tilted, cutout, get_test_parameter_set, write_results, get_results, normalize


def test_3d_backprop_phase_real():
    myframe = sys._getframe()
    myname = myframe.f_code.co_name
    print("running ", myname)
    sino, angles = create_test_sino_3d()
    parameters = get_test_parameter_set(2)
    # reference
    rref = list()
    for p in parameters:
        fref = odtbrain._Back_3D.backpropagate_3d(sino, angles, padval=0,
                                                  dtype=np.float64, onlyreal=True, **p)
        rref.append(cutout(fref))
    dataref = np.array(rref).flatten().view(float)
    
    
    r = list()
    for p in parameters:
        f = odtbrain._Back_3D_tilted.backpropagate_3d_tilted(sino, angles, padval=0,
                                               dtype=np.float64, onlyreal=True, **p)
        r.append(cutout(f))
    data = np.array(r).flatten().view(float)
    assert np.allclose(data, dataref)


def test_3d_backprop_pad():
    myframe = sys._getframe()
    myname = myframe.f_code.co_name
    print("running ", myname)
    sino, angles = create_test_sino_3d()
    parameters = get_test_parameter_set(2)
    # reference
    rref = list()
    for p in parameters:
        fref = odtbrain._Back_3D.backpropagate_3d(sino, angles, padval=None,
                                                  dtype=np.float64, onlyreal=False, **p)
        rref.append(cutout(fref))
    dataref = np.array(rref).flatten().view(float)
    
    
    r = list()
    for p in parameters:
        f = odtbrain._Back_3D_tilted.backpropagate_3d_tilted(sino, angles, padval=None,
                                               dtype=np.float64, onlyreal=False, **p)
        r.append(cutout(f))
    data = np.array(r).flatten().view(float)
    
    assert np.allclose(data, dataref)


def test_3d_backprop_plane_rotation():
    """
    A very soft test to check if planar rotation works fine
    in the reconstruction with tilted angles.
    """
    myframe = sys._getframe()
    myname = myframe.f_code.co_name
    print("running ", myname)
    
    parameters = get_test_parameter_set(1)
    results = []

    # These are specially selected angles that don't give high results.
    # Probably due to phase-wrapping, errors >2 may appear. Hence, we
    # call it a soft test. 
    tilts = [1.1, 0.0, 0.234, 2.80922, -.29, 9.87]
    
    for angz in tilts:
        sino, angles = create_test_sino_3d_tilted(tilt_plane=angz, A=21)
        rotmat = np.array([ 
                           [np.cos(angz), -np.sin(angz), 0],
                           [np.sin(angz),  np.cos(angz), 0],
                           [0           ,             0, 1],
                           ])
        # rotate `tilted_axis` onto the y-z plane.
        tilted_axis = np.dot(rotmat, [0,1,0])
        
        rref = list()
        for p in parameters:
            fref = odtbrain._Back_3D_tilted.backpropagate_3d_tilted(sino, angles, padval=None,
                                                      tilted_axis=tilted_axis, padding=(False, False),
                                                      dtype=np.float64, onlyreal=False, **p)
            rref.append(cutout(fref))
        data = np.array(rref).flatten().view(float)
        results.append(data)
    
    for ii in np.arange(len(results)):
        assert np.allclose(results[ii], results[ii-1], atol=.2, rtol=.2)
    

if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
This file contains miscellaneous methods that are used by
example scripts.
"""
from __future__ import print_function


import os
from os.path import dirname, join, exists, isdir
import tarfile

import warnings

datadir = "data"
webloc = "https://github.com/RI-imaging/ODTbrain/raw/master/examples/data"

def dl_file(url, dest, chunk_size=6553):
    """
    Download `url` to `dest`.
    """
    import urllib3
    http = urllib3.PoolManager()
    r = http.request('GET', url, preload_content=False)
    with open(dest, 'wb') as out:
        while True:
            data = r.read(chunk_size)
            if data is None or len(data)==0:
                break
            out.write(data)
    r.release_conn()


def get_file(fname):
    """
    Return the full path to a basename. If the file does not exist
    in the current directory or in subdirectory `datadir`, try to 
    download it from the public GitHub repository.
    """
    # download location
    dlloc = join(dirname(__file__), datadir)
    if exists(dlloc):
        if isdir(dlloc):
            pass
        else:
            raise OSError("Must be directory: "+dlloc)
    else:
        os.mkdir(dlloc)
        
    
    # find the file
    foundloc = None
    
    # Search possible file locations
    possloc = [dirname(__file__), dlloc]  

    for pl in possloc:
        if exists(join(pl, fname)):
            foundloc = join(pl, fname)
            break
                
    if foundloc is None:
        # Download file with urllib2.urlopen
        print("Attempting to download file {} from {} to {}.".
              format(fname, webloc, dlloc))
        try:
            dl_file(url=join(webloc, fname),
                    dest=join(dlloc, fname))
        except:
            warnings.warn("Download failed: "+fname)
            raise
        else:
            foundloc = join(dlloc, fname)
    
    if foundloc is None:
        raise OSError("Could not obtain file: "+fname)
    
    return foundloc


def load_tar_lzma_data(afile):
    """
    Load FDTD data from a .tar.lzma file.
    """
    try:
        import lzma
    except ImportError:
        try:
            from backports import lzma
        except ImportError:
            print("Please install lzma with 'pip install backports.lzma'")
    import numpy as np

    # open lzma file
    with open(afile, "rb") as l:
        data = lzma.decompress(l.read())
    # write tar file
    with open(afile[:-5], "wb") as t:
        t.write(data)
    # open tar file
    fields_real = []
    fields_imag = []
    phantom = []
    parms = {}
    
    with tarfile.open(afile[:-5], "r") as t:
        members = t.getmembers()
        members.sort(key=lambda x: x.name)
        
        for m in members:
            n = m.name
            f = t.extractfile(m)
            if n.startswith("fdtd_info"):
                for l in f.readlines():
                    l = l.decode()
                    if l.count("=") == 1:
                        key, val = l.split("=")
                        parms[key.strip()] = float(val.strip())
            elif n.startswith("phantom"):
                phantom.append(np.loadtxt(f))
            elif n.startswith("field"):
                if n.endswith("imag.txt"):
                    fields_imag.append(np.loadtxt(f))
                elif n.endswith("real.txt"):
                    fields_real.append(np.loadtxt(f))

    phantom = np.array(phantom)
    sino = np.array(fields_real)+1j*np.array(fields_imag)
    angles = np.linspace(0, 2*np.pi, sino.shape[0], endpoint=False)
    
    return sino, angles, phantom, parms


#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests 1D angular weights.
"""
from __future__ import division, print_function

import numpy as np
import os
from os.path import abspath, basename, dirname, join, split, exists
import platform
import sys
import warnings
import zipfile

# Add parent directory to beginning of path variable
DIR = dirname(abspath(__file__))
sys.path = [split(DIR)[0]] + sys.path

import odtbrain
import odtbrain._Back_2D

from common_methods import create_test_sino_2d, get_test_parameter_set, write_results, get_results


def test_angle_offset():
    """
    Tests if things are still correct when there is a 2PI offset in the angles.
    """
    myframe = sys._getframe()
    myname = myframe.f_code.co_name
    print("running ", myname)
    sino, angles = create_test_sino_2d()
    parameters = get_test_parameter_set(2)
    # reference
    r1 = []
    for p in parameters:
        f1 = odtbrain._Back_2D.backpropagate_2d(sino, angles, weight_angles=False, **p)
        r1.append(f1)
    # with offset
    angles[::2] += 2*np.pi*np.arange(angles[::2].shape[0])
    r2 = []
    for p in parameters:
        f2 = odtbrain._Back_2D.backpropagate_2d(sino, angles, weight_angles=False, **p)    
        r2.append(f2)
    # with offset and weights
    r3 = []
    for p in parameters:
        f3 = odtbrain._Back_2D.backpropagate_2d(sino, angles, weight_angles=True, **p)    
        r3.append(f3)
    assert np.allclose(np.array(r1).flatten().view(float),
                       np.array(r2).flatten().view(float))
    assert np.allclose(np.array(r2).flatten().view(float),
                       np.array(r3).flatten().view(float))


def test_angle_swap():
    """
    Test if everything still works, when angles are swapped.
    """
    myframe = sys._getframe()
    myname = myframe.f_code.co_name
    print("running ", myname)
    sino, angles = create_test_sino_2d()
    # remove elements so that we can see that weighting works
    angles = angles[:-2]
    sino = sino[:-2,:]
    parameters = get_test_parameter_set(2)
    # reference
    r1 = []
    for p in parameters:
        f1 = odtbrain._Back_2D.backpropagate_2d(sino, angles, weight_angles=True, **p)
        r1.append(f1)
    # change order of angles
    order = np.argsort(angles % .5) 
    angles = angles[order]
    sino = sino[order,:]
    r2 = []
    for p in parameters:
        f2 = odtbrain._Back_2D.backpropagate_2d(sino, angles, weight_angles=True, **p)    
        r2.append(f2)
    assert np.allclose(np.array(r1).flatten().view(float),
                       np.array(r2).flatten().view(float))


if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()
    

    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests backpropagation algorithm
"""
from __future__ import division, print_function

import numpy as np
import os
from os.path import abspath, basename, dirname, join, split, exists
import platform
import sys
import warnings
import zipfile

# Add parent directory to beginning of path variable
DIR = dirname(abspath(__file__))
sys.path = [split(DIR)[0]] + sys.path

import odtbrain
import odtbrain._Back_3D_tilted


def test_simple_sphere():
    """
    simple geometrical tests
    """
    angles = np.array([0, np.pi/2, np.pi])
    axes = [[1,0,0], [0,1,0], [0,0,1], [0,1,1], [1,0,1], [1,1,1]]

    results = []
    
    for tilted_axis in axes:
        angle_coords = odtbrain._Back_3D_tilted.sphere_points_from_angles_and_tilt(angles, tilted_axis)
        results.append(angle_coords)
    
    s2 = 1/np.sqrt(2)
    correct = np.array([ [[1,0,0], [1,0,0], [1,0,0]],
                         [[0,0,1], [1,0,0], [0,0,-1]],
                         [[0,0,1], [0,0,1], [0,0,1]],
                         [[0,0,1], [s2, .5, .5], [0,1,0]],
                         [[s2,0,s2], [s2,0,s2], [s2,0,s2]],
                         [[s2,0,s2], [0.87965281125489458,s2/3*2,0.063156230327168605], [s2/3,s2/3*4,s2/3]],
                        ])
    
    assert np.allclose(correct.flatten(), np.array(results).flatten())
    
if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()
    
    import matplotlib.pylab as plt
    from mpl_toolkits.mplot3d import Axes3D
    from matplotlib.patches import FancyArrowPatch
    from mpl_toolkits.mplot3d import proj3d

    class Arrow3D(FancyArrowPatch):
        def __init__(self, xs, ys, zs, *args, **kwargs):
            FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
            self._verts3d = xs, ys, zs
    
        def draw(self, renderer):
            xs3d, ys3d, zs3d = self._verts3d
            xs, ys, zs = proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
            self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
            FancyArrowPatch.draw(self, renderer)

    axes = [[0,1,0], [0,1,0.1], [0,1,-1], [1,0.1,0]]
    colors = ["k", "blue", "red", "green"]
    angles = np.linspace(0, 2*np.pi, 100)

    fig = plt.figure(figsize=(10,10))
    ax = fig.add_subplot(111, projection='3d')    
    for i in range(len(axes)):
        tilted_axis = axes[i]
        color = colors[i]
        tilted_axis = np.array(tilted_axis)
        tilted_axis = tilted_axis/np.sqrt(np.sum(tilted_axis**2)) 
        
        angle_coords = odtbrain._Back_3D_tilted.sphere_points_from_angles_and_tilt(angles, tilted_axis)
        
        u,v,w = tilted_axis
        a = Arrow3D([0,u],[0,v],[0,w], mutation_scale=20, lw=1, arrowstyle="-|>", color=color)
        ax.add_artist(a)
        ax.scatter(angle_coords[:,0], angle_coords[:,1], angle_coords[:,2], c=color, marker='o')
    
    radius=1
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_zlabel('Z')
    ax.set_xlim(-radius*1.5, radius*1.5)
    ax.set_ylim(-radius*1.5, radius*1.5)
    ax.set_zlim(-radius*1.5, radius*1.5)
    plt.tight_layout()

    plt.show()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Utility functions for ODTbrain
"""
from __future__ import division, print_function

import numpy as np


def compute_angle_weights_1d(angles):
    """
    Compute the weight for each angle according to the distance between its
    neighbors.
    
    Parameters
    ----------
    angles : 1d ndarray of length A
        Angles in radians
        
    Returns
    -------
    weights : 1d ndarray of length A
        The weights for each angle
    
    Notes
    -----
    To compute the weights, the angles are set modulo PI, not modulo 2PI.
    This reduces artifacts when the angular coverage is between PI and 2PI
    but does not affect the result when the angles cover the full 2PI interval.
    """
    # copy and modulo np.pi
    # This is an array with values in [0, np.pi)
    angles = (angles.flatten() - angles.min()) % (np.pi) 
    # sort the array
    sortargs = np.argsort(angles)
    sortangl = angles[sortargs]
    # compute weights for sorted angles
    da = (np.roll(sortangl, -1) - np.roll(sortangl, 1)) % (np.pi)
    weights = da/np.sum(da)*da.shape[0]
    
    unsortweights = np.zeros_like(weights)
    # Sort everything back where it belongs
    unsortweights[sortargs] = weights
    return unsortweights


#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests backpropagation algorithm
"""
from __future__ import division, print_function

import numpy as np
import os
from os.path import abspath, basename, dirname, join, split, exists
import platform
import sys
import warnings
import zipfile

# Add parent directory to beginning of path variable
DIR = dirname(abspath(__file__))
sys.path = [split(DIR)[0]] + sys.path

import odtbrain
import odtbrain._Back_3D_tilted

from common_methods import create_test_sino_2d, get_test_parameter_set, write_results, get_results


def test_rotate_points_to_axis():
    # rotation of axis itself always goes to y-axis
    rot1 = odtbrain._Back_3D_tilted.rotate_points_to_axis(points=[[1,2,3]], axis=[1,2,3])   
    assert rot1[0][0] < 1e-14
    assert rot1[0][2] < 1e-14
    rot2 = odtbrain._Back_3D_tilted.rotate_points_to_axis(points=[[-3,.6,.1]], axis=[-3,.6,.1])   
    assert rot2[0][0] < 1e-14
    assert rot2[0][2] < 1e-14
    
    sq2 = np.sqrt(2)
    # rotation to 45deg about x
    points=[[0,0,1], [1,0,0], [1,1,0]]
    rot3 = odtbrain._Back_3D_tilted.rotate_points_to_axis(points=points, axis=[0,1,1])
    assert np.allclose(rot3[0], [0,1/sq2,1/sq2])
    assert np.allclose(rot3[1], [1,0,0])
    assert np.allclose(rot3[2], [1,1/sq2,-1/sq2])

    # rotation to 45deg about y
    points=[[0,0,1], [1,0,0], [0,-1,0]]
    rot4 = odtbrain._Back_3D_tilted.rotate_points_to_axis(points=points, axis=[1,0,1])
    assert np.allclose(rot4[0], [-.5,1/sq2,.5])
    assert np.allclose(rot4[1], [.5,1/sq2,-.5])
    assert np.allclose(rot4[2], [1/sq2,0,1/sq2])
    
    # Visualization
    #plt, Arrow3D = setup_mpl()
    #fig = plt.figure(figsize=(10,10))
    #ax = fig.add_subplot(111, projection='3d')    
    #for vec in points:
    #    u,v,w = vec
    #    a = Arrow3D([0,u],[0,v],[0,w], mutation_scale=20, lw=1, arrowstyle="-|>", color="k")
    #    ax.add_artist(a)
    #for vec in rot4:
    #    u,v,w = vec
    #    a = Arrow3D([0,u],[0,v],[0,w], mutation_scale=20, lw=1, arrowstyle="-|>", color="b")
    #    ax.add_artist(a)
    #radius=1
    #ax.set_xlabel('X')
    #ax.set_ylabel('Y')
    #ax.set_zlabel('Z')
    #ax.set_xlim(-radius*1.5, radius*1.5)
    #ax.set_ylim(-radius*1.5, radius*1.5)
    #ax.set_zlim(-radius*1.5, radius*1.5)
    #plt.tight_layout()
    #plt.show()

    # rotation to -90deg about z
    points=[[0,0,1], [1,0,1], [1,-1,0]]
    rot4 = odtbrain._Back_3D_tilted.rotate_points_to_axis(points=points, axis=[1,0,0])
    assert np.allclose(rot4[0], [0,0,1])
    assert np.allclose(rot4[1], [0,1,1])
    assert np.allclose(rot4[2], [1,1,0])

    # negative axes
    # In this case, everything is rotated in the y-z plane
    # (this case is not physical for tomogrpahy)
    points=[[0,0,1], [1,0,0], [1,-1,0]]
    rot4 = odtbrain._Back_3D_tilted.rotate_points_to_axis(points=points, axis=[0,-1,0])
    assert np.allclose(rot4[0], [0,0,-1])
    assert np.allclose(rot4[1], [1,0,0])
    assert np.allclose(rot4[2], [1,1,0])
    

def test_rotation_matrix_from_point():
    """
    `rotation_matrix_from_point` generates a matrix that rotates a point at
    [0,0,1] to the position of the argument of the method.
    """
    sq2 = np.sqrt(2)
    # identity
    m1 = odtbrain._Back_3D_tilted.rotation_matrix_from_point([0,0,1])
    assert np.allclose(np.dot(m1, [1,2,3]), [1,2,3])
    assert np.allclose(np.dot(m1, [-3,.5,-.6]), [-3,.5,-.6])

    # simple
    m2 = odtbrain._Back_3D_tilted.rotation_matrix_from_point([0,1,1])
    assert np.allclose(np.dot(m2, [0,0,1]), [0,1/sq2,1/sq2])
    assert np.allclose(np.dot(m2, [0,1,1]), [0,sq2,0])
    assert np.allclose(np.dot(m2, [1,0,0]), [1,0,0])
    
    # negative
    m3 = odtbrain._Back_3D_tilted.rotation_matrix_from_point([-1,1,0])
    assert np.allclose(np.dot(m3, [1,0,0]), [0,0,-1])
    assert np.allclose(np.dot(m3, [0,1,1]), [0,sq2,0])
    assert np.allclose(np.dot(m3, [0,-1/sq2,-1/sq2]), [0,-1,0])    
    assert np.allclose(np.dot(m3, [0,1/sq2,-1/sq2]), [-1,0,0])
    assert np.allclose(np.dot(m3, [0,-1/sq2,1/sq2]), [1,0,0])    



def setup_mpl():
    import matplotlib.pylab as plt
    from mpl_toolkits.mplot3d import Axes3D
    from matplotlib.patches import FancyArrowPatch
    from mpl_toolkits.mplot3d import proj3d

    class Arrow3D(FancyArrowPatch):
        def __init__(self, xs, ys, zs, *args, **kwargs):
            FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
            self._verts3d = xs, ys, zs
    
        def draw(self, renderer):
            xs3d, ys3d, zs3d = self._verts3d
            xs, ys, zs = proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
            self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
            FancyArrowPatch.draw(self, renderer)

    return plt, Arrow3D


if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()
    

    import scipy
    import scipy.ndimage
    import spimagine
    plt, Arrow3D = setup_mpl()

    # Testarray
    N=50
    A=41
    proj = np.zeros((N,N,N))
    #proj[(N-1)/2,(N-1)/2,:(N-1)/2] = np.abs(np.linspace(-10,0,(N-1)/2))
    proj[(N)/2,(N)/2,:(N)/2] = np.abs(np.linspace(-10,1,(N)/2))
    
    # By default, the rotational axis in _Back_3D_tilted is the y-axis.
    # Define a rotational axis with a slight offset in x and in z.
    axis = np.array([0.0,1,.1])
    axis /= np.sqrt(np.sum(axis**2))
    
    # Now, obtain the 3D angles that are equally distributed on the unit
    # sphere and correspond to the positions of projections that we would
    # measure.
    angles = np.linspace(0, 2*np.pi, A, endpoint=False)
    #angles = np.array([0, np.pi/2, np.pi])
    # The first point in that array will be in the x-z-plane.
    points = odtbrain._Back_3D_tilted.sphere_points_from_angles_and_tilt(angles, axis)
    
    # The following steps are exactly those that are used in 
    # odtbrain._Back_3D_tilted.backpropagate_3d_tilted
    # to perform 3D reconstruction with tilted angles.
    u, v, w = axis
    theta = np.arccos(v)
    
    # We need three rotations.
    
    # IMPROTANT:
    # We perform the reconstruction such that the rotational axis
    # is equal to the y-axis! This is easier than implementing a
    # rotation about the rotational axis and tilting with theta
    # before and afterwards.
    
    # This is the rotation that tilts the projection in the
    # direction of the rotation axis (new y-axis). 
    Rtilt = np.array([
                   [1,             0,              0],
                   [0, np.cos(theta),  np.sin(theta)],
                   [0,-np.sin(theta),  np.cos(theta)],
                   ])
    
    out = np.zeros((N,N,N))
    
    vectors = []
    
    for ang, pnt in zip(angles, points):
        Rcircle = np.array([
                       [np.cos(ang),  0, np.sin(ang)],
                       [0           , 1,           0],
                       [-np.sin(ang), 0, np.cos(ang)],
                       ])

        DR = np.dot(Rtilt, Rcircle)
        
        # pnt are already rotated by R1
        vectors.append(np.dot(Rtilt, pnt))
        
        # We need to give this rotation the correct offset
        c = 0.5*np.array(proj.shape)
        offset=c-c.dot(DR.T)
        rotate = scipy.ndimage.interpolation.affine_transform(
                        proj, DR, offset=offset,
                        mode="constant", cval=0, order=2)
        proj *= 0.98
        out += rotate
    
    # visualize the axes
    out[0,0,0] = np.max(out)    #origin
    out[-1,0,0] = np.max(out)/2 # x
    out[0,-1,0] = np.max(out)/3 # z
    # show arrows pointing at projection directions (should form cone aligned with y)
    fig = plt.figure(figsize=(10,10))
    ax = fig.add_subplot(111, projection='3d')    
    for vec in vectors:
        u,v,w = vec
        a = Arrow3D([0,u],[0,v],[0,w], mutation_scale=20, lw=1, arrowstyle="-|>")
        ax.add_artist(a)
    
    radius=1
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_zlabel('Z')
    ax.set_xlim(-radius*1.5, radius*1.5)
    ax.set_ylim(-radius*1.5, radius*1.5)
    ax.set_zlim(-radius*1.5, radius*1.5)
    plt.tight_layout()
    plt.show()

    # show backprojection of test volume (should be cone aligned with y)
    spimagine.volshow(out)
    import IPython
    IPython.embed()    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests backpropagation algorithm
"""
from __future__ import division, print_function, unicode_literals

import numpy as np
import os
from os.path import abspath, basename, dirname, join, split, exists
from scipy.ndimage import rotate
import sys
import warnings
import zipfile

# Add parent directory to beginning of path variable
DIR = dirname(abspath(__file__))
sys.path = [split(DIR)[0]] + sys.path


def create_test_sino_2d(A=9, N=22, max_phase=5.0,
                        ampl_range=(1.0, 1.0)):
    """
    Creates 2D test sinogram for optical diffraction tomography.
    The sinogram is generated from a Gaussian that is shifted
    according to the rotational position of a non-centered
    object.
    
    Parameters
    ----------
    A : int
        Number of angles of the sinogram.
    N : int
        Size of one acquisition.
    max_phase : float
        Phase normalization. If this is greater than
        2PI, then it also tests the unwrapping
        capabilities of the reconstruction algorithm.
    ampl_range : tuple of floats
        Determines the min/max range of the amplitude values.
        Equal values means constant amplitude.
    """
    # initiate array
    resar = np.zeros((A, N), dtype=np.complex128)
    # 2pi coverage
    angles = np.linspace(0, 2*np.pi, A, endpoint=False)
    # x-values of Gaussain
    x = np.linspace(-N/2, N/2, N, endpoint=True)
    # SD of Gaussian
    dev = np.sqrt(N/2)
    # Off-centered rotation:
    off = N/7
    for ii in range(A):
        # Gaussian distribution sinogram
        x0 = np.cos(angles[ii])*off
        phase = np.exp(-(x-x0)**2/dev**2)
        phase = normalize(phase, vmax=max_phase)
        if ampl_range[0] == ampl_range[1]:
            # constant amplitude
            ampl = ampl_range[0]
        else:
            # ring
            ampldev = dev/5
            amploff = off*.3
            ampl1 = np.exp(-(x-x0-amploff)**2/ampldev**2)
            ampl2 = np.exp(-(x-x0+amploff)**2/ampldev**2)
            ampl = ampl1+ampl2
            ampl = normalize(ampl, vmin=ampl_range[0], vmax=ampl_range[1])
        resar[ii] = ampl*np.exp(1j*phase)
    return resar, angles


def create_test_sino_3d(A=9, Nx=22, Ny=22, max_phase=5.0,
                        ampl_range=(1.0,1.0)):
    """
    Creates 3D test sinogram for optical diffraction tomography.
    The sinogram is generated from a Gaussian that is shifted
    according to the rotational position of a non-centered
    object. The simulated rotation is about the second (y)/[1]
    axis.
    
    Parameters
    ----------
    A : int
        Number of angles of the sinogram.
    Nx : int
        Size of the first axis.
    Ny : int
        Size of the second axis.
    max_phase : float
        Phase normalization. If this is greater than
        2PI, then it also tests the unwrapping
        capabilities of the reconstruction algorithm.
    ampl_range : tuple of floats
        Determines the min/max range of the amplitude values.
        Equal values means constant amplitude.
    
    Returns
    """
    # initiate array
    resar = np.zeros((A, Ny, Nx), dtype=np.complex128)
    # 2pi coverage
    angles = np.linspace(0, 2*np.pi, A, endpoint=False)
    # x-values of Gaussain
    x = np.linspace(-Nx/2, Nx/2, Nx, endpoint=True).reshape(1,-1)
    y = np.linspace(-Ny/2, Ny/2, Ny, endpoint=True).reshape(-1,1)
    # SD of Gaussian
    dev = min(np.sqrt(Nx/2), np.sqrt(Ny/2))
    # Off-centered rotation  about second axis:
    off = Nx/7
    for ii in range(A):
        # Gaussian distribution sinogram
        x0 = np.cos(angles[ii])*off
        phase = np.exp(-(x-x0)**2/dev**2) * np.exp(-(y)**2/dev**2)
        phase = normalize(phase, vmax=max_phase)
        if ampl_range[0] == ampl_range[1]:
            # constant amplitude
            ampl = ampl_range[0]
        else:
            # ring
            ampldev = dev/5
            amploff = off*.3
            ampl1 = np.exp(-(x-x0-amploff)**2/ampldev**2)
            ampl2 = np.exp(-(x-x0+amploff)**2/ampldev**2)
            ampl = ampl1+ampl2
            ampl = normalize(ampl, vmin=ampl_range[0], vmax=ampl_range[1])
        resar[ii] = ampl*np.exp(1j*phase)
    return resar, angles


def create_test_sino_3d_tilted(A=9, Nx=22, Ny=22, max_phase=5.0,
                               ampl_range=(1.0,1.0),
                               tilt_plane=0.0):
    """
    Creates 3D test sinogram for optical diffraction tomography.
    The sinogram is generated from a Gaussian that is shifted
    according to the rotational position of a non-centered
    object. The simulated rotation is about the second (y)/[1]
    axis.
    
    Parameters
    ----------
    A : int
        Number of angles of the sinogram.
    Nx : int
        Size of the first axis.
    Ny : int
        Size of the second axis.
    max_phase : float
        Phase normalization. If this is greater than
        2PI, then it also tests the unwrapping
        capabilities of the reconstruction algorithm.
    ampl_range : tuple of floats
        Determines the min/max range of the amplitude values.
        Equal values means constant amplitude.
    tilt_plane : float
        Rotation tilt offset [rad].
    
    Returns
    """
    # initiate array
    resar = np.zeros((A, Ny, Nx), dtype=np.complex128)
    # 2pi coverage
    angles = np.linspace(0, 2*np.pi, A, endpoint=False)
    # x-values of Gaussain
    x = np.linspace(-Nx/2, Nx/2, Nx, endpoint=True).reshape(1,-1)
    y = np.linspace(-Ny/2, Ny/2, Ny, endpoint=True).reshape(-1,1)
    # SD of Gaussian
    dev = min(np.sqrt(Nx/2), np.sqrt(Ny/2))
    # Off-centered rotation  about second axis:
    off = Nx/7
    for ii in range(A):
        # Gaussian distribution sinogram
        x0 = np.cos(angles[ii])*off
        phase = np.exp(-(x-x0)**2/dev**2) * np.exp(-(y)**2/dev**2)
        phase = normalize(phase, vmax=max_phase)
        if ampl_range[0] == ampl_range[1]:
            # constant amplitude
            ampl = np.ones((Nx, Ny))*ampl_range[0]
        else:
            # ring
            ampldev = dev/5
            amploff = off*.3
            ampl1 = np.exp(-(x-x0-amploff)**2/ampldev**2)
            ampl2 = np.exp(-(x-x0+amploff)**2/ampldev**2)
            ampl = ampl1+ampl2
            ampl = normalize(ampl, vmin=ampl_range[0], vmax=ampl_range[1])

        # perform in-plane rotation
        ampl = rotate(ampl, np.rad2deg(tilt_plane), reshape=False, cval=1)
        phase = rotate(phase, np.rad2deg(tilt_plane), reshape=False, cval=0)
        resar[ii] = ampl*np.exp(1j*phase)

    return resar, angles


def cutout(a):
    """ cut out circle/sphere from 2D/3D square/cubic array
    """
    x = np.arange(a.shape[0])
    c = a.shape[0] / 2
    
    if len(a.shape) == 2:
        x = x.reshape(-1,1)
        y = x.reshape(1,-1)
        zero = ((x-c)**2 + (y-c)**2) < c**2
    elif len(a.shape) == 3:
        x = x.reshape(-1,1,1)
        y = x.reshape(1,-1,1)
        z = x.reshape(1,-1,1)
        zero = ((x-c)**2 + (y-c)**2 + (z-c)**2) < c**2
    else:
        raise ValueError("Cutout array must have dimension 2 or 3!")
    a *= zero
    #tool.arr2im(a, scale=True).save("test.png")
    return a


def get_test_parameter_set(set_number=1):
    res = 2.1
    lD = 0
    nm = 1.333
    parameters = []
    for i in range(set_number):
        parameters.append({"res" : res,
                           "lD" : lD,
                           "nm" : nm})
        res += .1
        lD += np.pi
        nm *= 1.01
    return parameters

def normalize(av, vmin=0, vmax=1):
    """
    normalize an array to the range vmin/vmax
    """
    if vmin == vmax:
        return np.ones_like(av)*vmin
    elif vmax < vmin:
        warnings.warn("swapping vmin and vmax, because vmax < vmin.")
        vmin, vmax = vmax, vmin
    
    norm_one = (av - np.min(av))/(np.max(av)-np.min(av))
    return norm_one * (vmax-vmin) + vmin


def get_results(frame):
    """ Get the results from the frame of a method """
    filen = frame.f_globals["__file__"]
    funcname = frame.f_code.co_name
    identifier = "{}__{}".format(filen.split("test_", 1)[1][:-3],
                                 funcname)
    wdir = join(dirname(abspath(__file__)), "data")
    text = join(wdir, identifier+".txt")
    zipf = join(wdir, identifier+".zip")
    if exists(text):
        pass
    elif exists(zipf):
        with zipfile.ZipFile(zipf) as arc:
            arc.extract(basename(text), wdir)
    else:
        raise ValueError("No reference found for test: {}".format(text))
    
    data = np.loadtxt(text)
    return data

def write_results(frame, r):
    """
    Used for writing the results to zip-files in the current directory.
    If put in the directory "data", these files will be used for tests.
    """
    data = np.array(r).flatten().view(float)
    filen = frame.f_globals["__file__"]
    funcname = frame.f_code.co_name
    identifier = "{}__{}".format(filen.split("test_", 1)[1][:-3],
                                 funcname)
    text = identifier+".txt"
    zipf = identifier+".zip"
    # remove existing files
    if exists(text):
        os.remove(text)
    if exists(zipf):
        os.remove(zipf)
    # save text
    np.savetxt(text, data, fmt="%.10f")
    # make zip
    with zipfile.ZipFile(zipf, "w", compression=zipfile.ZIP_DEFLATED) as arc:
        arc.write(text)
    os.remove(text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import division, print_function

import ctypes
import gc
import multiprocessing as mp
import numexpr as ne
import numpy as np
from numpy import cos, sin

import pyfftw
import scipy.ndimage
import warnings

from ._Back_3D import _ncores, _np_float64, _verbose
from . import util
import odtbrain


def estimate_major_rotation_axis(loc):
    """ 
    For a list of points on the unit sphere, estimate the main
    rotational axis and return a list of angles that correspond
    to the rotational position for each point.
    
    
    """
    #TODO:
    raise NotImplementedError("estimation of rotational axis not implemented.") 


def norm_vec(vector):
    """
    Normalizes a vector to one.
    """
    assert len(vector)==3
    v = np.array(vector)
    return v/np.sqrt(np.sum(v**2))


def rotate_points_to_axis(points, axis):
    """ Rotate all points of a list, such that `axis==[0,1,0]`.
    
    This is accomplished by rotating in the x-z-plane by phi into the
    y-z-plane, then rotation in the y-z-plane by theta up to [0,1,0],
    and finally rotating back in the x-z-plane by -phi.
    
    Parameters
    ----------
    points : list-like with elements of length 3
        The Cartesian points. These should be in the same format as
        produced by `sphere_points_from_angles_and_tilt`.
    axis : list-like, length 3
        The reference axis that will be used to determine the
        rotation angle of the points. The points will be rotated
        about the origin such that `axis` matches [0,1,0].
    
    Returns
    -------
    rotated_points : np.ndarray of shape (N,3)
        The rotated points.
    """
    axis = norm_vec(axis)
    u, v, w = axis
    points = np.array(points)
    # Determine the rotational angle in the x-z plane
    phi = np.arctan2(u, w)
    # Determine the tilt angle w.r.t. the y-axis
    theta = np.arccos(v)
    
    # Negative rotation about y-axis
    Rphi = np.array([
                     [np.cos(phi), 0, -np.sin(phi)],
                     [0          , 1,            0],
                     [np.sin(phi), 0,  np.cos(phi)],
                     ])

    # Negative rotation about x-axis
    Rtheta = np.array([
                       [1,             0,              0],
                       [0, np.cos(theta),  np.sin(theta)],
                       [0,-np.sin(theta),  np.cos(theta)],
                       ])

    DR1 = np.dot(Rtheta, Rphi)
    # Rotate back by -phi such that effective rotation was only
    # towards [0,1,0].
    DR = np.dot(Rphi.T, DR1)
    rotpoints = np.zeros((len(points), 3))
    for ii, pnt in enumerate(points):
        rotpoints[ii] = np.dot(DR, pnt)

    # For visualiztaion:
    #import matplotlib.pylab as plt
    #from mpl_toolkits.mplot3d import Axes3D
    #from matplotlib.patches import FancyArrowPatch
    #from mpl_toolkits.mplot3d import proj3d
    #
    #class Arrow3D(FancyArrowPatch):
    #    def __init__(self, xs, ys, zs, *args, **kwargs):
    #        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
    #        self._verts3d = xs, ys, zs
    #
    #    def draw(self, renderer):
    #        xs3d, ys3d, zs3d = self._verts3d
    #        xs, ys, zs = proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
    #        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
    #        FancyArrowPatch.draw(self, renderer)
    #
    #fig = plt.figure(figsize=(10,10))
    #ax = fig.add_subplot(111, projection='3d')    
    #for vec in rotpoints:
    #    u,v,w = vec
    #    a = Arrow3D([0,u],[0,v],[0,w], mutation_scale=20, lw=1, arrowstyle="-|>")
    #    ax.add_artist(a)
    #
    #radius=1
    #ax.set_xlabel('X')
    #ax.set_ylabel('Y')
    #ax.set_zlabel('Z')
    #ax.set_xlim(-radius*1.5, radius*1.5)
    #ax.set_ylim(-radius*1.5, radius*1.5)
    #ax.set_zlim(-radius*1.5, radius*1.5)
    #plt.tight_layout()
    #plt.show()

    return rotpoints


def rotation_matrix_from_point(point, ret_inv=False):
    """ Compute rotation matrix to go from [0,0,1] to `point`.
    
    First, the matrix rotates to in the polar direction. Then,
    a rotation about the y-axis is performed to match the
    azimuthal angle in the x-z-plane.
    
    This rotation matrix is required for the correct 3D orientation
    of the backpropagated projections.

    Parameters
    ----------
    points : list-like, length 3
        The coordinates of the point in 3D.
    ret_inv : bool
        Also return the inverse of the rotation matrix. The inverse
        is required for :func:`scipy.ndimage.interpolation.affine_transform`
        which maps the output coordinates to the input coordinates.
        
    Returns
    -------
    Rmat [, Rmat_inv] : 3x3 ndarrays
        The rotation matrix that rotates [0,0,1] to `point` and
        optionally its inverse.
    """
    x, y, z = point
    # azimuthal angle
    phi = np.arctan2(x, z)
    # angle in polar direction (negative)
    theta = -np.arctan2(y, np.sqrt(x**2+z**2))
    
    # Rotation in polar direction
    Rtheta = np.array([
                       [1,             0,              0],
                       [0, np.cos(theta), -np.sin(theta)],
                       [0, np.sin(theta),  np.cos(theta)],
                       ])

    # rotation in x-z-plane
    Rphi = np.array([
                     [np.cos(phi), 0, -np.sin(phi)],
                     [0          , 1,            0],
                     [np.sin(phi), 0, np.cos(phi)],
                     ])

    
    D = np.dot(Rphi, Rtheta)
    # The inverse of D
    Dinv = np.dot(Rtheta.T, Rphi.T)

    if ret_inv:
        return D, Dinv
    else:
        return D


def rotation_matrix_from_point_planerot(point, plane_angle, ret_inv=False):
    """ Compute rotation matrix to go from [0,0,1] to `point`,
    while taking into account the tilted axis of rotation.
    
    First, the matrix rotates to in the polar direction. Then,
    a rotation about the y-axis is performed to match the
    azimuthal angle in the x-z-plane.
    
    This rotation matrix is required for the correct 3D orientation
    of the backpropagated projections.

    Parameters
    ----------
    points : list-like, length 3
        The coordinates of the point in 3D.
    axis : list-like, length 3
        The coordinates of the point in 3D.
    ret_inv : bool
        Also return the inverse of the rotation matrix. The inverse
        is required for :func:`scipy.ndimage.interpolation.affine_transform`
        which maps the output coordinates to the input coordinates.
        
    Returns
    -------
    Rmat [, Rmat_inv] : 3x3 ndarrays
        The rotation matrix that rotates [0,0,1] to `point` and
        optionally its inverse.
    """
    # These matrices are correct if there is no tilt of the
    # rotational axis within the detector plane (x-y).
    D, Dinv = rotation_matrix_from_point(point, ret_inv=True)
    
    # We need an additional rotation about the z-axis to correct
    # for the tilt for all the the other cases.
    angz = plane_angle
    
    Rz =  np.array([
                    [np.cos(angz), -np.sin(angz), 0],
                    [np.sin(angz),  np.cos(angz), 0],
                    [0,             0,            1],
                    ])
    
    DR = np.dot(D, Rz)
    DRinv = np.dot(Rz.T, Dinv)
    
    if ret_inv:
        return DR, DRinv
    else:
        return DR


def sphere_points_from_angles_and_tilt(angles, tilted_axis):
    """
    For a given tilt of the rotational axis `tilted_axis`, compute
    the points on a unit sphere that correspond to the distribution
    `angles` along the great circle about this axis.
    
    Parameters
    ----------
    angles : 1d ndarray
        The angles that will be distributed on the great circle.
    tilted_axis : list of length 3
        The tilted axis of rotation that determines the great
        circle.
    
    Notes
    -----
    The reference axis is always [0,1,0].
    `theta` is the azimuthal angle measured down from the y-axis.
    `phi` is the polar angle in the x-z plane measured from z towards x.

    """
    assert len(angles.shape) == 1 
    ## Normalize tilted axis.
    tilted_axis = norm_vec(tilted_axis)
    [u, v, w] = tilted_axis
    ## Initial distribution of points about great circle (x-z).
    newang = np.zeros((angles.shape[0], 3), dtype=float)
    # We subtract angles[0], because in step (a) we want that
    # newang[0]==[0,0,1]. This only works if we actually start
    # at that point.
    newang[:,0] = np.sin(angles-angles[0])
    newang[:,2] = np.cos(angles-angles[0])
    
    ## Compute rotational angles w.r.t. [0,1,0].
    # - Draw a unit sphere with the y-axis pointing up and the
    #   z-axis pointing right
    # - The rotation of `tilted_axis` can be described by two
    #   separate rotations. We will use these two angles:
    #   (a) Rotation from y=1 within the y-z plane: theta
    #       This is the rotation that is critical for data
    #       reconstruction. If this angle is zero, then we
    #       have a rotational axis in the imaging plane. If
    #       this angle is PI/2, then our sinogram consists
    #       of a rotating image and 3D reconstruction is
    #       impossible. This angle is counted from the y-axis
    #       onto the x-z plane.
    #   (b) Rotation in the x-z plane: phi
    #       This angle is responsible for matching up the angles
    #       with the correct sinogram images. If this angle is zero,
    #       then the projection of the rotational axis onto the
    #       x-y plane is aligned with the y-axis. If this angle is
    #       PI/2, then the axis and its projection onto the x-y
    #       plane are identical. This angle is counted from the
    #       positive z-axis towards the positive x-axis. By default,
    #       angles[0] is the point that touches the great circle
    #       that lies in the x-z plane. angles[1] is the next point
    #       towards the x-axis if phi==0.
    
    # (a) This angle is the azimuthal angle theta measured from the
    #     y-axis.
    theta = np.arccos(v)
    
    # (b) This is the polar angle measured in the x-z plane starting
    #     at the x-axis and measured towards the positive z-axis.
    phi = np.arctan2(u, w)
    
    ## Determine the projection points on the unit sphere.
    # The resulting circle meets the x-z-plane at phi, and
    # is tilted by theta w.r.t. the y-axis.
    
    # (a) Create a tilted data set. This is achieved in 3 steps.

    # a1) Determine radius of tilted circle and get the centered
    #     circle with a smaller radius.
    rtilt = np.cos(theta)
    newang *= rtilt

    # a2) Rotate this circle about the x-axis by theta
    #     (right-handed/counter-clockwise/basic/elemental rotation)
    Rx = np.array([  
               [1,          0,           0],
               [0, cos(theta), -sin(theta)],
               [0, sin(theta),  cos(theta)]
               ])
    for ii in range(newang.shape[0]):
        newang[ii] = np.dot(Rx, newang[ii])

    # a3) Shift newang such that newang[0] is located at (0,0,1)
    newang = newang - (newang[0] - np.array([0,0,1])).reshape(1,3)

    # (b) Rotate the entire thing with phi about the y-axis
    #     (right-handed/counter-clockwise/basic/elemental rotation)
    Ry = np.array([  
                   [ cos(phi), 0, sin(phi)],
                   [        0, 1,        0],
                   [-sin(phi), 0, cos(phi)]
                   ])
    
    for jj in range(newang.shape[0]):
        newang[jj] = np.dot(Ry, newang[jj])

    # For visualiztaion:
    #import matplotlib.pylab as plt
    #from mpl_toolkits.mplot3d import Axes3D
    #from matplotlib.patches import FancyArrowPatch
    #from mpl_toolkits.mplot3d import proj3d
    #
    #class Arrow3D(FancyArrowPatch):
    #    def __init__(self, xs, ys, zs, *args, **kwargs):
    #        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
    #        self._verts3d = xs, ys, zs
    #
    #    def draw(self, renderer):
    #        xs3d, ys3d, zs3d = self._verts3d
    #        xs, ys, zs = proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
    #        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
    #        FancyArrowPatch.draw(self, renderer)
    #
    #fig = plt.figure(figsize=(10,10))
    #ax = fig.add_subplot(111, projection='3d')    
    #for vec in newang:
    #    u,v,w = vec
    #    a = Arrow3D([0,u],[0,v],[0,w], mutation_scale=20, lw=1, arrowstyle="-|>")
    #    ax.add_artist(a)
    #
    #radius=1
    #ax.set_xlabel('X')
    #ax.set_ylabel('Y')
    #ax.set_zlabel('Z')
    #ax.set_xlim(-radius*1.5, radius*1.5)
    #ax.set_ylim(-radius*1.5, radius*1.5)
    #ax.set_zlim(-radius*1.5, radius*1.5)
    #plt.tight_layout()
    #plt.show()

    return newang


def backpropagate_3d_tilted(uSin, angles, res, nm, lD=0,
                     tilted_axis=[0, 1, 0],
                     coords=None, weight_angles=True, onlyreal=False,
                     offset_alpha=0, offset_beta=0,
                     padding=(True, True), padfac=1.75, padval=None,
                     intp_order=2, dtype=_np_float64,
                     num_cores=_ncores,
                     save_memory=False,
                     copy=True,
                     jmc=None, jmm=None,
                     verbose=_verbose):
    u""" 3D backpropagation with the Fourier diffraction theorem
    Three-dimensional diffraction tomography reconstruction
    algorithm for scattering of a plane wave
    :math:`u_0(\mathbf{r}) = u_0(x,y,z)` 
    by a dielectric object with refractive index
    :math:`n(x,y,z)`.
    
    This method implements the 3D backpropagation algorithm with
    a rotational axis that is tilted by :math:`\\theta_\mathrm{tilt}`
    w.r.t. the imaging plane [3]_

    .. math::
        f(\mathbf{r}) = 
            -\\frac{i k_\mathrm{m}}{2\pi}
            \\sum_{j=1}^{N} \! \Delta \phi_0 D_{-\phi_j}^\mathrm{tilt} \!\!
            \\left \{
            \\text{FFT}^{-1}_{\mathrm{2D}}
            \\left \{
            \\left| k_\mathrm{Dx} \cdot \cos \\theta_\mathrm{tilt}\\right|  
            \\frac{\\text{FFT}_{\mathrm{2D}} \\left \{
            u_{\mathrm{B},\phi_j}(x_\mathrm{D}, y_\mathrm{D}) \\right \}}
            {u_0(l_\mathrm{D})}
            \exp \! \\left[i k_\mathrm{m}(M - 1) \cdot
            (z_{\phi_j}-l_\mathrm{D}) \\right]
            \\right \} 
            \\right \}

    with a modified rotational operator :math:`D_{-\phi_j}^\mathrm{tilt}`
    and a different filter in Fourier space 
    :math:`|k_\mathrm{Dx} \cdot \cos \\theta_\mathrm{tilt}|` when compared
    to :func:`backpropagate_3d`.
    
    
    .. versionadded:: 0.1.2
    
    Parameters
    ----------
    uSin : (A, Ny, Nx) ndarray
        Three-dimensional sinogram of plane recordings
        :math:`u_{\mathrm{B}, \phi_j}(x_\mathrm{D}, y_\mathrm{D})`
        divided by the incident plane wave :math:`u_0(l_\mathrm{D})`
        measured at the detector.
    angles : ndarray of shape (A,3) or 1D array of length A
        If the shape is (A,3), then `angles` consists of vectors
        on the unit sphere that correspond to the direction
        of illumination and acquisition (s₀). If the shape is (A,),
        then `angles` is  a one-dimensional array of angles in radians
        that determines the angular position :math:`\phi_j`.
        In both cases, `tilted_axis` must be set according to the
        tilt of the rotational axis.
    res : float
        Vacuum wavelength of the light :math:`\lambda` in pixels.
    nm : float
        Refractive index of the surrounding medium :math:`n_\mathrm{m}`.
    lD : float
        Distance from center of rotation to detector plane 
        :math:`l_\mathrm{D}` in pixels.
    tilted_axis : list of floats
        The coordinates [x, y, z] on a unit sphere representing the
        tilted axis of rotation. The default is (0,1,0),
        which corresponds to a rotation about the y-axis and
        follows the behavior of :func:`odtbrain.backpropagate_3d`.
    coords : None [(3, M) ndarray]
        Only compute the output image at these coordinates. This
        keyword is reserved for future versions and is not
        implemented yet.
    weight_angles : bool
        If `True`, weights each backpropagated projection with a factor
        proportional to the angular distance between the neighboring
        projections. 
        
        .. math::
            \Delta \phi_0 \\longmapsto \Delta \phi_j = \\frac{\phi_{j+1} - \phi_{j-1}}{2}
        
        This currently only works when `angles` has the shape (A,).
    onlyreal : bool
        If `True`, only the real part of the reconstructed image
        will be returned. This saves computation time.
    padding : tuple of bool
        Pad the input data to the second next power of 2 before
        Fourier transforming. This reduces artifacts and speeds up
        the process for input image sizes that are not powers of 2.
        The default is padding in x and y: `padding=(True, True)`.
        For padding only in x-direction (e.g. for cylindrical
        symmetries), set `padding` to `(True, False)`. To turn off
        padding, set it to `(False, False)`.
    padfac : float
        Increase padding size of the input data. A value greater
        than one will trigger padding to the second-next power of
        two. For example, a value of 1.75 will lead to a padded
        size of 256 for an initial size of 144, whereas for it will
        lead to a padded size of 512 for an initial size of 150.
        Values geater than 2 are allowed. This parameter may
        greatly increase memory usage!
    padval : float
        The value used for padding. This is important for the Rytov
        approximation, where an approximat zero in the phase might
        translate to 2πi due to the unwrapping algorithm. In that
        case, this value should be a multiple of 2πi. 
        If `padval` is `None`, then the edge values are used for
        padding (see documentation of :func:`numpy.pad`).
    order : int between 0 and 5
        Order of the interpolation for rotation.
        See :func:`scipy.ndimage.interpolation.affine_transform` for details.
    dtype : dtype object or argument for :func:`numpy.dtype`
        The data type that is used for calculations (float or double).
        Defaults to `numpy.float`.
    num_cores : int
        The number of cores to use for parallel operations. This value
        defaults to the number of cores on the system.
    save_memory : bool
        Saves memory at the cost of longer computation time.
        
        .. versionadded:: 0.1.5
        
    copy : bool
        Copy input sinogram `uSin` for data processing. If `copy`
        is set to `False`, then `uSin` will be overridden.
        
        .. versionadded:: 0.1.5
        
    jmc, jmm : instance of :func:`multiprocessing.Value` or `None`
        The progress of this function can be monitored with the 
        :mod:`jobmanager` package. The current step `jmc.value` is
        incremented `jmm.value` times. `jmm.value` is set at the 
        beginning.
    verbose : int
        Increment to increase verbosity.


    Returns
    -------
    f : ndarray of shape (Nx, Ny, Nx), complex if `onlyreal==False`
        Reconstructed object function :math:`f(\mathbf{r})` as defined
        by the Helmholtz equation.
        :math:`f(x,z) = 
        k_m^2 \\left(\\left(\\frac{n(x,z)}{n_m}\\right)^2 -1\\right)`


    See Also
    --------
    odt_to_ri : conversion of the object function :math:`f(\mathbf{r})` 
        to refractive index :math:`n(\mathbf{r})`.


    Notes
    -----
    This implementation can deal with projection angles that are not
    distributed along a circle  about the rotational axis. If there are
    slight deviations from this circle, simply pass the 3D rotational
    positions instead of the 1D angles to the `angles` argument. In
    principle, this should improve the reconstruction. The general
    problem here is that the backpropagation algorithm requires a
    ramp filter in Fourier space that is oriented perpendicular to the
    rotational axis. If the sample does not rotate about a single axis,
    then a 1D parametric representation of this rotation must be found
    to correctly determine the filter in Fourier space. Such a
    parametric representation could e.g. be a spiral between the poles
    of the unit sphere (but this kind of rotation is probably difficult
    to implement experimentally).

    If you have input images with rectangular shape, e.g. Nx!=Ny and the
    rotational axis deviates by approximately PI/2 from the axis (0,1,0), then
    data might get cropped in the reconstruction volume. You can avoid that by
    rotating your input data and the rotational axis by PI/2. For instance, change
    `tilted_axis` from [1,0,0] to [0,1,0] and `np.rot90` the sinogram images.
    
    Do not use the parameter `lD` in combination with the Rytov
    approximation - the propagation is not correctly described.
    Instead, numerically refocus the sinogram prior to converting
    it to Rytov data (using e.g. :func:`odtbrain.sinogram_as_rytov`)
    with a numerical focusing algorithm (available in the Python
    package :py:mod:`nrefocus`).
    """
    ne.set_num_threads(num_cores)

    if copy:
        uSin = uSin.copy()
        angles = angles.copy()

    # `tilted_axis` is required for several things:
    # 1. the filter |kDx*v + kDy*u| with (u,v,w)==tilted_axis
    # 2. the alignment of the rotational axis with the y-axis
    # 3. the determination of the point coordinates if only
    #    angles in radians are given.
    
    # For (1) we need the exact axis that corresponds to our input data.
    # For (2) and (3) we need `tilted_axis_yz` (see below) which is the
    # axis `tilted_axis` rotated in the detector plane such that its
    # projection onto the detector coincides with the y-axis.
    
    # Normalize input axis
    tilted_axis=norm_vec(tilted_axis)

    # `tilted_axis_yz` is computed by performing the inverse rotation in
    # the x-y plane with `angz`. We will again use `angz` in the transform
    # within the for-loop to rotate each projection according to its 
    # acquisition angle.
    angz = np.arctan2(tilted_axis[0], tilted_axis[1])
    rotmat = np.array([ 
                       [np.cos(angz), -np.sin(angz), 0],
                       [np.sin(angz),  np.cos(angz), 0],
                       [0           ,             0, 1],
                       ])
    # rotate `tilted_axis` onto the y-z plane.
    tilted_axis_yz = norm_vec(np.dot(rotmat, tilted_axis))
    
    A = angles.shape[0]
    angles = np.squeeze(angles) # Allow shapes (A,1)
    assert angles.shape in [(A,), (A,3)], "`angles` must have shape (A,) or (A,3)!"
    # jobmanager
    if jmm is not None:
        jmm.value = A + 2
    
    if len(angles.shape) == 1:
        if weight_angles:
            weights = util.compute_angle_weights_1d(angles).reshape(-1,1,1)
        # compute the 3D points from tilted axis
        angles = sphere_points_from_angles_and_tilt(angles, tilted_axis_yz)
    else:
        if weight_angles:
            warnings.warn("3D angular weighting not yet supported!")
            weights = 1

        # normalize and rotate angles
        for ii in range(angles.shape[0]):
            #angles[ii] = norm_vec(angles[ii]) #-> not correct
            # instead rotate like `tilted_axis` onto the y-z plane.
            angles[ii] = norm_vec(np.dot(rotmat, angles[ii]))
    
    # check for dtype
    dtype = np.dtype(dtype)
    assert dtype.name in ["float32", "float64"], "dtype must be float32 or float64!"

    assert num_cores <= _ncores, "`num_cores` must not exceed number " +\
                                 "of physical cores: {}".format(_ncores)

    assert uSin.dtype == np.complex128, "uSin dtype must be complex128."

    dtype_complex = np.dtype("complex{}".format(
        2 * int(dtype.name.strip("float"))))

    # set ctype
    ct_dt_map = {np.dtype(np.float32): ctypes.c_float,
                 np.dtype(np.float64): ctypes.c_double
                 }

    assert len(uSin.shape) == 3, "Input data `uSin` must have shape (A,Ny,Nx)."
    assert len(uSin) == A, "`len(angles)` must be  equal to `len(uSin)`."
    assert len(list(padding)) == 2, "Parameter `padding` must be boolean tuple of length 2!"
    assert np.array(padding).dtype is np.dtype(bool), "Parameter `padding` must be boolean tuple."
    assert coords is None, "Setting coordinates is not yet supported."

    # Cut-Off frequency
    # km [1/px]
    km = (2 * np.pi * nm) / res
    # The notation in the our optical tomography script for
    # a wave propagating to the right is:
    #
    #    u0(x) = exp(ikx)
    #
    # However, in physics usually we use the other sign convention:
    #
    #    u0(x) = exp(-ikx)
    #
    # In order to be consistent with programs like Meep or our
    # scattering script for a dielectric cylinder, we want to use the
    # latter sign convention.
    # This is not a big problem. We only need to multiply the imaginary
    # part of the scattered wave by -1.
    sinogram = uSin

    if weight_angles:
        sinogram *= weights

    # lengths of the input data
    (la, lny, lnx) = sinogram.shape
    ln = lnx

    # We do a zero-padding before performing the Fourier transform.
    # This gets rid of artifacts due to false periodicity and also
    # speeds up Fourier transforms of the input image size is not
    # a power of 2.
    # transpose so we can call resize correctly

    orderx = max(64., 2**np.ceil(np.log(lnx * padfac) / np.log(2)))
    ordery = max(64., 2**np.ceil(np.log(lny * padfac) / np.log(2)))

    if padding[0]:
        padx = orderx - lnx
    else:
        padx = 0
    if padding[1]:
        pady = ordery - lny
    else:
        pady = 0

    # Apply a Fourier filter before projecting the sinogram slices.
    # Resize image to next power of two for fourier analysis
    # (reduces artifacts).
    padyl = np.int(np.ceil(pady / 2))
    padyr = np.int(pady - padyl)
    padxl = np.int(np.ceil(padx / 2))
    padxr = np.int(padx - padxl)


    #TODO: This padding takes up a lot of memory. Move it to a separate
    # for loop or to the main for-loop.
    if padval is None:
        sino = np.pad(sinogram, ((0, 0), (padyl, padyr), (padxl, padxr)),
                      mode="edge")
        if verbose > 0:
            print("......Padding with edge values.")
    else:
        sino = np.pad(sinogram, ((0, 0), (padyl, padyr), (padxl, padxr)),
                      mode="linear_ramp",
                      end_values=(padval,))
        if verbose > 0:
            print("......Verifying padding value: {}".format(padval))

    # save memory
    del sinogram
    
    if verbose > 0:
        print("......Image size (x,y): {}x{}, padded: {}x{}".format(
            lnx, lny, sino.shape[2], sino.shape[1]))

    # zero-padded length of sinogram.
    (lA, lNy, lNx) = sino.shape  # @UnusedVariable
    lNz = ln


    # Ask for the filter. Do not include zero (first element).
    #
    # Integrals over ϕ₀ [0,2π]; kx [-kₘ,kₘ]
    #   - double coverage factor 1/2 already included
    #   - unitary angular frequency to unitary ordinary frequency
    #     conversion performed in calculation of UB=FT(uB).
    #
    # f(r) = -i kₘ / ((2π)² a₀)                 (prefactor)
    #      * iiint dϕ₀ dkx dky                  (prefactor)
    #      * |kx|                               (prefactor)
    #      * exp(-i kₘ M lD )                   (prefactor)
    #      * UBϕ₀(kx)                           (dependent on ϕ₀)
    #      * exp( i (kx t⊥ + kₘ (M - 1) s₀) r ) (dependent on ϕ₀ and r)
    # (r and s₀ are vectors. The last term contains a dot-product)
    #
    # kₘM = sqrt( kₘ² - kx² - ky² )
    # t⊥  = (  cos(ϕ₀), ky/kx, sin(ϕ₀) )
    # s₀  = ( -sin(ϕ₀), 0    , cos(ϕ₀) )
    #
    # The filter can be split into two parts
    #
    # 1) part without dependence on the z-coordinate
    #
    #        -i kₘ / ((2π)² a₀)
    #      * iiint dϕ₀ dkx dky
    #      * |kx|
    #      * exp(-i kₘ M lD )
    #
    # 2) part with dependence of the z-coordinate
    #
    #        exp( i (kx t⊥ + kₘ (M - 1) s₀) r )
    #
    # The filter (1) can be performed using the classical filter process
    # as in the backprojection algorithm.
    #
    #

    # if lNx != lNy:
    #    raise NotImplementedError("Input data must be square shaped!")

    # Corresponding sample frequencies
    fx = np.fft.fftfreq(lNx)  # 1D array
    fy = np.fft.fftfreq(lNy)  # 1D array
    # kx is a 1D array.
    kx = 2 * np.pi * fx
    ky = 2 * np.pi * fy
    # Differentials for integral
    dphi0 = 2 * np.pi / A
    # We will later multiply with phi0.
    #               a, y, x
    kx = kx.reshape(1, 1, -1)
    ky = ky.reshape(1, -1, 1)
    # Low-pass filter:
    # less-than-or-equal would give us zero division error.
    filter_klp = (kx**2 + ky**2 < km**2)

    # Filter M so there are no nans from the root
    M = 1. / km * np.sqrt((km**2 - kx**2 - ky**2) * filter_klp)

    prefactor = -1j * km / (2 * np.pi)
    prefactor *= dphi0
    # Also filter the prefactor, so nothing outside the required
    # low-pass contributes to the sum.
    # The filter is now dependent on the rotational position of the
    # specimen. We have to include information from the angles.
    # We want to estimate the rotational axis for every frame. We 
    # do that by computing the cross-product of the vectors in
    # angles from the current and previous image.
    
    u, v, _w = tilted_axis
    filterabs = np.abs(kx*v+ky*u) * filter_klp
    # new in version 0.1.4:
    # We multiply by the factor (M-1) instead of just (M)
    # to take into account that we have a scattered
    # wave that is normalized by u0.
    prefactor *= np.exp(-1j * km * (M-1) * lD)
    # Perform filtering of the sinogram,
    # save memory by in-place operations
    #projection = np.fft.fft2(sino, axes=(-1,-2)) * prefactor
    # Flag is "estimate":
    #   specifies that, instead of actual measurements of different
    #   algorithms, a simple heuristic is used to pick a (probably
    #   sub-optimal) plan quickly. With this flag, the input/output
    #   arrays are not overwritten during planning.

    # Byte-aligned arrays
    temp_array = pyfftw.n_byte_align_empty(sino[0].shape, 16, dtype_complex)

    myfftw_plan = pyfftw.FFTW(temp_array, temp_array, threads=num_cores,
                              flags=["FFTW_ESTIMATE"], axes=(0,1))


    if jmc is not None:
        jmc.value += 1

    for p in range(len(sino)):
        # this overwrites sino
        temp_array[:] = sino[p, :, :]
        myfftw_plan.execute()
        sino[p, :, :] = temp_array[:]

    temp_array, myfftw_plan

    projection = sino
    projection[:] *= prefactor / (lNx * lNy)
    projection[:] *= filterabs

    # save memory
    del prefactor, filter_klp
    #
    #
    # filter (2) must be applied before rotation as well
    # exp( i (kx t⊥ + kₘ (M - 1) s₀) r )
    #
    # kₘM = sqrt( kₘ² - kx² - ky² )
    # t⊥  = (  cos(ϕ₀), ky/kx, sin(ϕ₀) )
    # s₀  = ( -sin(ϕ₀), 0    , cos(ϕ₀) )
    #
    # This filter is effectively an inverse Fourier transform
    #
    # exp(i kx xD) exp(i ky yD) exp(i kₘ (M - 1) zD )
    #
    # xD =   x cos(ϕ₀) + z sin(ϕ₀)
    # zD = - x sin(ϕ₀) + z cos(ϕ₀)

    # Everything is in pixels
    center = lNz / 2.0
    #x = np.linspace(-centerx, centerx, lNx, endpoint=False)
    #x = np.arange(lNx) - center + .5
    # Meshgrid for output array
    #zv, yv, xv = np.meshgrid(x,x,x)
    #               z, y, x
    #xv = x.reshape( 1, 1,-1)
    #yv = x.reshape( 1,-1, 1)

    #z = np.arange(ln) - center + .5
    z = np.linspace(-center, center, lNz, endpoint=False)
    zv = z.reshape(-1, 1, 1)

    #              z, y, x
    Mp = M.reshape(lNy, lNx)

    # filter2 = np.exp(1j * zv * km * (Mp - 1))
    f2_exp_fac = 1j* km * (Mp - 1)
    if save_memory:
        # compute filter2 later
        pass
    else:
        # compute filter2 now
        filter2 = ne.evaluate("exp(factor * zv)",
                              local_dict={"factor": f2_exp_fac,
                                          "zv":zv})
        # occupies some amount of ram, but yields faster
        # computation later
        #filter2[0].size*len(filter2)*128/8/1024**3

    if jmc is not None:
        jmc.value += 1

    #                               a, z, y,  x
    #projection = projection.reshape(la, 1, lNy, lNx)
    projection = projection.reshape(la, lNy, lNx)

    # This frees comparatively few data
    del M
    #del Mp

    # Prepare complex output image
    if onlyreal:
        outarr = np.zeros((ln, lny, lnx), dtype=dtype)
    else:
        outarr = np.zeros((ln, lny, lnx), dtype=dtype_complex)

    # Create plan for fftw:
    inarr = pyfftw.n_byte_align_empty((lNy, lNx), 16, dtype_complex)
    #inarr[:] = (projection[0]*filter2)[0,:,:]
    # plan is "patient":
    #    FFTW_PATIENT is like FFTW_MEASURE, but considers a wider range
    #    of algorithms and often produces a “more optimal” plan
    #    (especially for large transforms), but at the expense of
    #    several times longer planning time (especially for large
    #    transforms).
    # print(inarr.flags)


    myifftw_plan = pyfftw.FFTW(inarr, inarr, threads=num_cores,
                               axes=(0,1),
                               direction="FFTW_BACKWARD",
                               flags=["FFTW_MEASURE"])


    #assert shared_array.base.base is shared_array_base.get_obj()
    shared_array_base = mp.Array(ct_dt_map[dtype], ln * lny * lnx)
    _shared_array = np.ctypeslib.as_array(shared_array_base.get_obj())
    _shared_array = _shared_array.reshape(ln, lny, lnx)

    # Initialize the pool with the shared array
    odtbrain._shared_array = _shared_array
    pool4loop = mp.Pool(processes=num_cores)

    # filtered projections in loop
    filtered_proj = np.zeros((ln, lny, lnx), dtype=dtype_complex)

    # Rotate all points such that we are effectively rotating everything
    # about the y-axis.
    angles = rotate_points_to_axis(points=angles, axis=tilted_axis_yz)

    for aa in np.arange(A):
        # A == la
        # projection.shape == (A, lNx, lNy)
        # filter2.shape == (ln, lNx, lNy)
        
        for p in range(len(zv)):
            if save_memory:
                # compute filter2 here;
                # this is comparatively slower than the other case
                ne.evaluate("exp(factor * zvp) * projectioni",
                            local_dict={"zvp":zv[p],
                                        "projectioni":projection[aa],
                                        "factor":f2_exp_fac},
                            out=inarr)
            else:
                # use universal functions
                np.multiply(filter2[p], projection[aa], out=inarr)
            myifftw_plan.execute()
            filtered_proj[p, :, :] = inarr[
                                           padyl:padyl + lny,
                                           padxl:padxl + lnx
                                          ]
        
        # The Cartesian axes in our array are ordered like this: [z,y,x]
        # However, the rotation matrix requires [x,y,z]. Therefore, we
        # need to np.transpose the first and last axis and also invert the
        # y-axis.
        fil_p_t = filtered_proj.transpose(2,1,0)[:,::-1,:]

        # get rotation matrix for this point and also rotate in plane
        _drot, drotinv = rotation_matrix_from_point_planerot(angles[aa],
                                                             plane_angle=angz,
                                                             ret_inv=True)
        
        ## apply offset required by affine_transform
        # The offset is only required for the rotation in
        # the x-z-plane.
        # This could be achieved like so:
        # The offset "-.5" assures that we are rotating about
        # the center of the image and not the value at the center
        # of the array (this is also what `scipy.ndimage.rotate` does.
        c = 0.5 * np.array(fil_p_t.shape) - .5
        offset = c - np.dot(drotinv, c)
        
        ## Perform rotation
        # We cannot split the inplace-rotation into multiple subrotations
        # as we did in _Back_3d_tilted.backpropagate_3d, because the rotation
        # axis is arbitrarily placed in the 3d array. Rotating single
        # slices does not yield the same result as rotating the entire
        # array. Instead of using affine_transform, map_coordinates might
        # be faster for multiple cores.
        
        # Also undo the axis transposition that we performed previously.

        outarr.real += scipy.ndimage.interpolation.affine_transform(
                                fil_p_t.real, drotinv,
                                offset=offset, mode="constant",
                                cval=0, order=intp_order).transpose(2,1,0)[:,::-1,:]

        if not onlyreal:
            outarr.imag += scipy.ndimage.interpolation.affine_transform(
                                    fil_p_t.imag, drotinv,
                                    offset=offset, mode="constant",
                                    cval=0, order=intp_order).transpose(2,1,0)[:,::-1,:]

        if jmc is not None:
            jmc.value += 1

    pool4loop.terminate()
    pool4loop.join()

    del _shared_array, inarr, odtbrain._shared_array
    del shared_array_base

    gc.collect()

    return outarr



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" 
2D cell phantom (FDTD simulation)
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
The *in silico* data set was created with the 
:abbr:`FDTD (Finite Difference Time Domain)` software `meep`_. The data
are 1D projections of a 2D refractive index phantom. The
reconstruction of the refractive index with the Rytov approximation
is in good agreement with the phantom that was used in the
simulation.

.. figure::  ../examples/backprop_from_fdtd_2d_repo.png
   :align:   center

   2D reconstruction from :abbr:`FDTD (Finite Difference Time Domain)`
   data created by `meep`_ simulations.

Download the :download:`full example <../examples/backprop_from_fdtd_2d.py>`.
If you are not running the example from the git repository, make sure the
file :download:`example_helper.py <../examples/example_helper.py>` is present
in the current directory.

.. _`meep`: http://ab-initio.mit.edu/wiki/index.php/Meep

"""
from __future__ import division, print_function

# All imports are moved to "__main__", because
# sphinx might complain about some imports (e.g. mpl).
if __name__ == "__main__":
    try:
        from example_helper import get_file
    except ImportError:
        print("Please make sure example_helper.py is available.")
        raise
    import matplotlib.pylab as plt
    import numpy as np
    from os.path import abspath, dirname, split, join
    import sys
    import zipfile
    
    # Add parent directory to beginning of path variable
    DIR = dirname(abspath(__file__))
    sys.path.insert(0, split(DIR)[0])
    
    import odtbrain as odt
    
    # use jobmanager if available
    try:
        import jobmanager as jm
        jm.decorators.decorate_module_ProgressBar(odt, 
                            decorator=jm.decorators.ProgressBarOverrideCount,
                            interval=.1)
    except:
        pass

    datazip = get_file("fdtd_2d_sino_A100_R13.zip")
    # Get simulation data
    arc = zipfile.ZipFile(datazip)

    angles = np.loadtxt(arc.open("fdtd_angles.txt"))
    phantom = np.loadtxt(arc.open("fdtd_phantom.txt"))
    sino_real = np.loadtxt(arc.open("fdtd_real.txt"))
    sino_imag = np.loadtxt(arc.open("fdtd_imag.txt"))
    sino = sino_real +1j*sino_imag
    # get nm, lD, res
    with arc.open("fdtd_info.txt") as info:
        cfg = {}
        for l in info.readlines():
            l = l.decode()
            if l.count("=") == 1:
                key, val = l.split("=")
                cfg[key.strip()] = float(val.strip())

    print("Example: Backpropagation from 2d FDTD simulations")
    print("Refractive index of medium:", cfg["nm"])
    print("Measurement position from object center:", cfg["lD"])
    print("Wavelength sampling:", cfg["res"])
    print("Performing backpropagation.")

    ## Apply the Rytov approximation
    sinoRytov = odt.sinogram_as_rytov(sino)

    ## perform backpropagation to obtain object function f
    f = odt.backpropagate_2d( uSin=sinoRytov,
                              angles=angles,
                              res=cfg["res"],
                              nm=cfg["nm"],
                              lD=cfg["lD"]*cfg["res"]
                              )

    ## compute refractive index n from object function
    n = odt.odt_to_ri(f, res=cfg["res"], nm=cfg["nm"])

    ## compare phantom and reconstruction in plot
    fig, axes = plt.subplots(1, 3, figsize=(12,4), dpi=300)
    
    axes[0].set_title("FDTD phantom")
    axes[0].imshow(phantom, vmin=phantom.min(), vmax=phantom.max())
    sino_phase = np.unwrap(np.angle(sino), axis=1)
    axes[0].set_xlabel("x")
    axes[0].set_ylabel("y")
    
    axes[1].set_title("phase sinogram")    
    axes[1].imshow(sino_phase, vmin=sino_phase.min(), vmax=sino_phase.max(),
                   aspect=sino.shape[1]/sino.shape[0],
                   cmap=plt.cm.coolwarm)  # @UndefinedVariable
    axes[1].set_xlabel("detector")
    axes[1].set_ylabel("angle [rad]")

    axes[2].set_title("reconstructed image")
    axes[2].imshow(n.real, vmin=phantom.min(), vmax=phantom.max())
    axes[2].set_xlabel("x")    
    axes[2].set_ylabel("y")

    # set y ticks for sinogram
    labels = np.linspace(0, 2*np.pi, len(axes[1].get_yticks()))
    labels = [ "{:.2f}".format(i) for i in labels ]
    axes[1].set_yticks(np.linspace(0, len(angles), len(labels)))
    axes[1].set_yticklabels(labels)

    plt.tight_layout()
   
    outname = join(DIR, "backprop_from_fdtd_2d.png")
    print("Creating output file:", outname)
    plt.savefig(outname)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
u""" Algorithms for scalar diffraction tomography.

This package provides reconstruction algorithms for diffraction
tomography in two and three dimensions.

Installation
------------
ODTbrain is a python library that is compatible to Python 2 and Python 3.
To install via the `Python package manager (PyPi)`_, simply issue the
following command.

    pip install odtbrain


The `FFTW3 library`_ and the scientific python packages
:py:mod:`numpy` and :py:mod:`scipy` are required by ODTbrain.
If the above command does not work, please refer to the 
installation instructions at the `GitHub repository`_.

.. _`FFTW3 library`: http://fftw.org
.. _`GitHub repository`: https://github.com/RI-imaging/ODTbrain
.. _`Python package manager (PyPi)`: https://pypi.python.org/pypi/odtbrain/


Theoretical background
----------------------
A detailed summary of the underlying theory is available in [1]_.

The Fourier diffraction theorem states, that the Fourier transform
:math:`\widehat{U}_{\mathrm{B},\phi_0}(\mathbf{k_\mathrm{D}})` of 
the scattered field :math:`u_\mathrm{B}(\mathbf{r_D})`, measured at 
a certain angle :math:`\phi_0`, is distributed along a circular arc 
(2D) or along a semi-spherical surface (3D) in Fourier space,
synthesizing the Fourier transform 
:math:`\widehat{F}(\mathbf{k})` of the object function 
:math:`f(\mathbf{r})` ([4]_, [5]_).

.. math::

   \widehat{F}(k_\mathrm{m}(\mathbf{s - s_0}))= 
        - \sqrt{\\frac{2}{\pi}}  \\frac{i k_\mathrm{m}}{a_0} 
        M \widehat{U}_{\mathrm{B},\phi_0}(\mathbf{k_\mathrm{D}}) 
        \exp \! \\left(-i k_\mathrm{m} M l_\mathrm{D} \\right)
    
In this notation, 
:math:`k_\mathrm{m}` is the wave number,
:math:`\mathbf{s_0}` is the norm vector pointing at :math:`\phi_0`,
:math:`M=\sqrt{1-s_\mathrm{x}^2}` (2D) and
:math:`M=\sqrt{1-s_\mathrm{x}^2-s_\mathrm{y}^2}` (3D)
enforces the spherical constraint, and
:math:`l_\mathrm{D}` is the distance from the center of the object
function :math:`f(\mathbf{r})` to the detector plane
:math:`\mathbf{r_D}`.


Fields of Application
---------------------
The algorithms presented here are based on the (scalar) Helmholtz
equation. Furthermore, the Born and Rytov approximations to the
scattered wave :math:`u(\mathbf{r})` are used to linearize the
problem for a straight-forward inversion.

The package is intended for optical diffraction
tomography to determine the refractive index of biological cells.
Because the Helmholtz equation is only an approximation to the
Maxwell equations, describing the propagation of light, 
:abbr:`FDTD (Finite Difference Time Domain)` simulations were performed
to test the reconstruction algorithms within this package.
The algorithms present in this package should also be valid for the
following cases, but have not been tested appropriately:

* tomographic measurements of absorbing materials (complex refractive 
  index :math:`n(\mathbf{r})`)

* ultrasonic diffraction tomography, which is correctly described by
  the Helmholtz equation

How to cite
-----------
If you use ODTbrain in a scientific publication, please cite
Müller et al., *BMC Bioinformatics* (2015) [2]_. 


"""
from ._Back_2D import backpropagate_2d, fourier_map_2d, sum_2d
from ._Back_3D import backpropagate_3d
from ._Back_3D_tilted import backpropagate_3d_tilted
from ._br import odt_to_ri, opt_to_ri, sinogram_as_radon, sinogram_as_rytov
from ._version import version as __version__
from ._version import longversion as __version_full__

__author__ = u"Paul Müller"
__license__ = "BSD (3 clause)"


# Shared variable used by 3D backpropagation
_shared_array = None


#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" 
Unevenly spaced angles (2D)
~~~~~~~~~~~~~~~~~~~~~~~~~~~
Angular weighting can significantly improve reconstruction quality
when the angular projections are sampled at non-equidistant
intervals [6]_. The *in silico* data set was created with the 
softare `miefield  <https://github.com/RI-imaging/miefield>`_.
The data are 1D projections of a non-centered cylinder of constant
refractive index 1.339 embedded in water with refractive index 1.333. 
The first column shows the used sinograms (missing angles are displayed
as zeros) that were created from the original sinogram with 250
projections. The second column shows the reconstruction without angular
weights and the third column shows the reconstruction with angular
weights. The keyword argument `weight_angle` was introduced in version
0.1.1. 

.. figure::  ../examples/backprop_2d_weights_angles_repo.png
   :align:   center

   Impact of angular weighting on backpropagation with the Rytov
   approximation. 

Download the :download:`full example <../examples/backprop_2d_weights_angles.py>`.
If you are not running the example from the git repository, make sure the
file :download:`example_helper.py <../examples/example_helper.py>` is present
in the current directory.

"""
from __future__ import division, print_function

# All imports are moved to "__main__", because
# sphinx might complain about some imports (e.g. mpl).
if __name__ == "__main__":
    try:
        from example_helper import get_file
    except ImportError:
        print("Please make sure example_helper.py is available.")
        raise
    import matplotlib.pylab as plt
    import numpy as np
    from os.path import abspath, dirname, split, join
    import sys
    import zipfile
    import unwrap
    
    # Add parent directory to beginning of path variable
    DIR = dirname(abspath(__file__))
    sys.path.insert(0, split(DIR)[0])
    
    import odtbrain as odt
    
    # use jobmanager if available
    try:
        import jobmanager as jm
        jm.decorators.decorate_module_ProgressBar(odt, 
                            decorator=jm.decorators.ProgressBarOverrideCount,
                            interval=.1)
    except:
        pass

    datazip = get_file("mie_2d_noncentered_cylinder_A250_R2.zip")
    
    # Get simulation data
    arc = zipfile.ZipFile(datazip)

    angles = np.loadtxt(arc.open("mie_angles.txt"))

    # sinogram computed with mie
    # computed with
    # miefield.GetSinogramCylinderRotation(radius, nmed, ncyl, lD, lC, size, A, res)
    sino_real = np.loadtxt(arc.open("sino_real.txt"))
    sino_imag = np.loadtxt(arc.open("sino_imag.txt"))
    sino = sino_real + 1j*sino_imag
    A, size = sino_real.shape

    # background sinogram computed with mie
    # computed with
    # miefield.GetSinogramCylinderRotation(radius, nmed, nmed, lD, lC, size, A, res)
    u0_real = np.loadtxt(arc.open("u0_real.txt"))
    u0_imag = np.loadtxt(arc.open("u0_imag.txt"))
    u0 = u0_real + 1j*u0_imag
    # create 2d array
    u0 = np.tile(u0, size).reshape(A,size).transpose()

    # background field necessary to compute initial born field
    # computed with
    # u0_single = mie.GetFieldCylinder(radius, nmed, nmed, lD, size, res)
    u0_single_real = np.loadtxt(arc.open("u0_single_real.txt"))
    u0_single_imag = np.loadtxt(arc.open("u0_single_real.txt"))
    u0_single = u0_single_real + 1j*u0_single_imag

    with arc.open("mie_info.txt") as info:
        cfg = {}
        for l in info.readlines():
            l = l.decode()
            if l.count("=") == 1:
                key, val = l.split("=")
                cfg[key.strip()] = float(val.strip())

    print("Example: Backpropagation from 2d FDTD simulations")
    print("Refractive index of medium:", cfg["nmed"])
    print("Measurement position from object center:", cfg["lD"])
    print("Wavelength sampling:", cfg["res"])
    print("Performing backpropagation.")

    
    # Set measurement parameters
    # Compute scattered field from cylinder
    radius = cfg["radius"] # wavelengths
    nmed = cfg["nmed"]
    ncyl = cfg["ncyl"]
    
    lD = cfg["lD"] # measurement distance in wavelengths
    lC = cfg["lC"] # displacement from center of image
    size = cfg["size"]
    res = cfg["res"] # px/wavelengths
    A = cfg["A"] # number of projections

    #phantom = np.loadtxt(arc.open("mie_phantom.txt"))
    x = np.arange(size)-size/2.0
    X,Y = np.meshgrid(x,x)
    rad_px = radius*res
    phantom = np.array(((Y-lC*res)**2+X**2)<rad_px**2, dtype=np.float)*(ncyl-nmed)+nmed

    u_sinR = odt.sinogram_as_rytov(sino/u0)
    
    # Rytov 200 projections    
    # remove 50 projections from total of 250 projections
    remove200 = np.argsort(angles % .002)[:50]
    angles200 = np.delete(angles, remove200, axis=0)
    u_sinR200 = np.delete(u_sinR, remove200, axis=0)
    ph200 = unwrap.unwrap(np.angle(sino/u0))
    ph200[remove200] = 0
    
    fR200 = odt.backpropagate_2d(u_sinR200, angles200, res, nmed, lD*res)
    nR200 = odt.odt_to_ri(fR200, res, nmed)
    fR200nw = odt.backpropagate_2d(u_sinR200, angles200, res, nmed, lD*res, weight_angles=False)
    nR200nw = odt.odt_to_ri(fR200nw, res, nmed)
    
    # Rytov 50 projections
    remove50 = np.argsort(angles % .002)[:200]
    angles50 = np.delete(angles, remove50, axis=0)
    u_sinR50 = np.delete(u_sinR, remove50, axis=0)
    ph50 = unwrap.unwrap(np.angle(sino/u0))
    ph50[remove50] = 0
    
    fR50 = odt.backpropagate_2d(u_sinR50, angles50, res, nmed, lD*res)
    nR50 = odt.odt_to_ri(fR50, res, nmed)    
    fR50nw = odt.backpropagate_2d(u_sinR50, angles50, res, nmed, lD*res, weight_angles=False)
    nR50nw = odt.odt_to_ri(fR50nw, res, nmed)
    
    # prepare plot
    
    kw_ri = {"vmin": np.min(np.array([phantom, nR50.real, nR200.real])),
             "vmax": np.max(np.array([phantom, nR50.real, nR200.real]))
             }

    kw_ph = {"vmin": np.min(np.array([ph200, ph50])),
             "vmax": np.max(np.array([ph200, ph50])),
             "cmap": plt.cm.coolwarm  # @UndefinedVariable
             }
    
    
    fig, axes = plt.subplots(2,3, figsize=(12,7), dpi=300)
    axes = np.array(axes).flatten()
    
    phmap = axes[0].imshow(ph200, **kw_ph)
    axes[0].set_title("Phase sinogram (200 proj.)")

    rimap = axes[1].imshow(nR200nw.real, **kw_ri)
    axes[1].set_title("RI without angular weights")

    axes[2].imshow(nR200.real, **kw_ri)
    axes[2].set_title("RI with angular weights")
    
    axes[3].imshow(ph50, **kw_ph)
    axes[3].set_title("Phase sinogram (50 proj.)")

    axes[4].imshow(nR50nw.real, **kw_ri)
    axes[4].set_title("RI without angular weights")

    axes[5].imshow(nR50.real, **kw_ri)
    axes[5].set_title("RI with angular weights")    

    # color bars
    cbkwargs = {"fraction": 0.045,
                "format":"%.3f"}
    plt.colorbar(phmap, ax=axes[0], **cbkwargs)
    plt.colorbar(phmap, ax=axes[3], **cbkwargs)
    plt.colorbar(rimap, ax=axes[1], **cbkwargs)
    plt.colorbar(rimap, ax=axes[2], **cbkwargs)
    plt.colorbar(rimap, ax=axes[5], **cbkwargs)
    plt.colorbar(rimap, ax=axes[4], **cbkwargs)

    
    plt.tight_layout()
    
    outname = join(DIR, "backprop_2d_weights_angles.png")
    print("Creating output file:", outname)
    plt.savefig(outname)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" 
2D non-centered cylinder (Mie theory)
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
The *in silico* data set was created with the 
softare `miefield  <https://github.com/RI-imaging/miefield>`_.
The data are 1D projections of a non-centered cylinder of constant
refractive index. The Born approximation is error-prone due to
a relatively large radius of the cylinder (30 wavelengths) and
a refractive index difference of 0.006 between cylinder and
surrounding medium. The reconstruction of the refractive index
with the Rytov approximation is in good agreement with the
input data. When only 50 projections are used for the reconstruction,
artifacts appear. These vanish when more projections are used for
the reconstruction.

.. figure::  ../examples/backprop_from_mie_2d_cylinder_noncentered_repo.png
   :align:   center

   Different reconstruction approaches of a
   cylinder from data computed with Mie theory.

Download the :download:`full example <../examples/backprop_from_mie_2d_cylinder_noncentered.py>`.
If you are not running the example from the git repository, make sure the
file :download:`example_helper.py <../examples/example_helper.py>` is present
in the current directory.

"""
from __future__ import division, print_function

# All imports are moved to "__main__", because
# sphinx might complain about some imports (e.g. mpl).

if __name__ == "__main__":
    try:
        from example_helper import get_file
    except ImportError:
        print("Please make sure example_helper.py is available.")
        raise
    import matplotlib.pylab as plt
    import numpy as np
    from os.path import abspath, dirname, split, join
    import sys
    import zipfile
    import unwrap
    
    # Add parent directory to beginning of path variable
    DIR = dirname(abspath(__file__))
    sys.path.insert(0, split(DIR)[0])
    
    import odtbrain as odt
    
    # use jobmanager if available
    try:
        import jobmanager as jm
        jm.decorators.decorate_module_ProgressBar(odt, 
                            decorator=jm.decorators.ProgressBarOverrideCount,
                            interval=.1)
    except:
        pass

    datazip = get_file("mie_2d_noncentered_cylinder_A250_R2.zip")
    
    # Get simulation data
    arc = zipfile.ZipFile(datazip)

    angles = np.loadtxt(arc.open("mie_angles.txt"))

    # sinogram computed with mie
    # computed with
    # miefield.GetSinogramCylinderRotation(radius, nmed, ncyl, lD, lC, size, A, res)
    sino_real = np.loadtxt(arc.open("sino_real.txt"))
    sino_imag = np.loadtxt(arc.open("sino_imag.txt"))
    sino = sino_real + 1j*sino_imag
    A, size = sino_real.shape

    # background sinogram computed with mie
    # computed with
    # miefield.GetSinogramCylinderRotation(radius, nmed, nmed, lD, lC, size, A, res)
    u0_real = np.loadtxt(arc.open("u0_real.txt"))
    u0_imag = np.loadtxt(arc.open("u0_imag.txt"))
    u0 = u0_real + 1j*u0_imag
    # create 2d array
    u0 = np.tile(u0, size).reshape(A,size).transpose()

    # background field necessary to compute initial born field
    # computed with
    # u0_single = mie.GetFieldCylinder(radius, nmed, nmed, lD, size, res)
    u0_single_real = np.loadtxt(arc.open("u0_single_real.txt"))
    u0_single_imag = np.loadtxt(arc.open("u0_single_real.txt"))
    u0_single = u0_single_real + 1j*u0_single_imag

    with arc.open("mie_info.txt") as info:
        cfg = {}
        for l in info.readlines():
            l=l.decode()
            if l.count("=") == 1:
                l = l.decode()
                key, val = l.split("=")
                cfg[key.strip()] = float(val.strip())

    print("Example: Backpropagation from 2d Mie simulations")
    print("Refractive index of medium:", cfg["nmed"])
    print("Measurement position from object center:", cfg["lD"])
    print("Wavelength sampling:", cfg["res"])
    print("Performing backpropagation.")
    
    # Set measurement parameters
    # Compute scattered field from cylinder
    radius = cfg["radius"] # wavelengths
    nmed = cfg["nmed"]
    ncyl = cfg["ncyl"]
    
    lD = cfg["lD"] # measurement distance in wavelengths
    lC = cfg["lC"] # displacement from center of image
    size = cfg["size"]
    res = cfg["res"] # px/wavelengths
    A = cfg["A"] # number of projections

    #phantom = np.loadtxt(arc.open("mie_phantom.txt"))
    x = np.arange(size)-size/2.0
    X,Y = np.meshgrid(x,x)
    rad_px = radius*res
    phantom = np.array(((Y-lC*res)**2+X**2)<rad_px**2, dtype=np.float)*(ncyl-nmed)+nmed

    # Born
    u_sinB = (sino/u0*u0_single-u0_single) #fake born
    fB = odt.backpropagate_2d(u_sinB, angles, res, nmed, lD*res)
    nB = odt.odt_to_ri(fB, res, nmed)
    
    # Rytov
    u_sinR = odt.sinogram_as_rytov(sino/u0)
    fR = odt.backpropagate_2d(u_sinR, angles, res, nmed, lD*res)
    nR = odt.odt_to_ri(fR, res, nmed)
    
    # Rytov 50
    u_sinR50 = odt.sinogram_as_rytov((sino/u0)[::5,:])
    fR50 = odt.backpropagate_2d(u_sinR50, angles[::5], res, nmed, lD*res)
    nR50 = odt.odt_to_ri(fR50, res, nmed)
    
    # Plot sinogram phase and amplitude
    ph = unwrap.unwrap(np.angle(sino/u0))
    
    am = np.abs(sino/u0)

    # prepare plot
    vmin = np.min(np.array([phantom, nB.real, nR50.real, nR.real]))
    vmax = np.max(np.array([phantom, nB.real, nR50.real, nR.real]))
    
    fig, axes = plt.subplots(2,3, figsize=(12,7), dpi=300)
    axes = np.array(axes).flatten()
    
    phantommap = axes[0].imshow(phantom, vmin=vmin, vmax=vmax)
    axes[0].set_title("phantom \n(non-centered cylinder)")
    
    amplmap = axes[1].imshow(am, cmap=plt.cm.gray)  # @UndefinedVariable
    axes[1].set_title("amplitude sinogram \n(background-corrected)")
    
    phasemap = axes[2].imshow(ph, cmap=plt.cm.coolwarm)  # @UndefinedVariable
    axes[2].set_title("phase sinogram [rad] \n(background-corrected)")
    
    axes[3].imshow(nB.real, vmin=vmin, vmax=vmax)
    axes[3].set_title("reconstruction (Born) \n(250 projections)")
    
    axes[4].imshow(nR50.real, vmin=vmin, vmax=vmax)
    axes[4].set_title("reconstruction (Rytov) \n(50 projections)")
    
    axes[5].imshow(nR.real, vmin=vmin, vmax=vmax)
    axes[5].set_title("reconstruction (Rytov) \n(250 projections)")
    
    # color bars
    cbkwargs = {"fraction": 0.045}
    plt.colorbar(phantommap, ax=axes[0], **cbkwargs)
    plt.colorbar(amplmap, ax=axes[1], **cbkwargs)
    plt.colorbar(phasemap, ax=axes[2], **cbkwargs)
    plt.colorbar(phantommap, ax=axes[3], **cbkwargs)
    plt.colorbar(phantommap, ax=axes[4], **cbkwargs)
    plt.colorbar(phantommap, ax=axes[5], **cbkwargs)
    
    plt.tight_layout()
    
    outname = join(DIR, "backprop_from_mie_2d_cylinder_noncentered.png")
    print("Creating output file:", outname)
    plt.savefig(outname)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
3D sphere (Mie theory)
~~~~~~~~~~~~~~~~~~~~~~
The *in silico* data set was created with the Mie calculation software
`GMM-field`_. The data consist of a two-dimensional projection of a
sphere with radius :math:`R=14\lambda`,
refractive index :math:`n_\mathrm{sph}=1.006`,
embedded in a medium of refractive index :math:`n_\mathrm{med}=1.0`
onto a detector which is :math:`l_\mathrm{D} = 20\lambda` away from the
center of the sphere.

If the package :mod:`nrefocus` is installed, a better reconstruction
is obtained by numerical autofocusing the detected field prior to the
refractive index reconstruction and then setting :math:`l_\mathrm{D}=0`
for the reconstruction with :func:`odtbrain.backpropagate_3d`.

.. figure::  ../examples/backprop_from_mie_3d_sphere_repo.png
   :align:   center

   3D reconstruction from Mie simulations of a perfect sphere using
   200 projections. Missing angle artifacts are visible along the 
   :math:`y`-axis due to the :math:`2\pi`-only coverage in 3D Fourier
   space.
   

Download the 
:download:`full example <../examples/backprop_from_mie_3d_sphere.py>`.
If you are not running the example from the git repository, make sure the
file :download:`example_helper.py <../examples/example_helper.py>` is present
in the current directory.


.. _`GMM-field`: https://code.google.com/p/scatterlib/wiki/Nearfield

"""

# TODO:
# - in reconstruction label axes according to wavelengths
# - use latex for axis labeling
from __future__ import division, print_function


# All imports are moved to "__main__", because
# sphinx might complain about some imports (e.g. mpl).
if __name__ == "__main__":
    try:
        from example_helper import get_file
    except ImportError:
        print("Please make sure example_helper.py is available.")
        raise
    
    import matplotlib.pylab as plt
    import numpy as np
    from os.path import abspath, dirname, split, join
    import sys
    import warnings
    import zipfile
    
    # Add parent directory to beginning of path variable
    DIR = dirname(abspath(__file__))
    sys.path.insert(0, split(DIR)[0])
    
    import odtbrain as odt
    
    # Use nrefocus package for autofocusing
    try:
        import nrefocus
    except:
        warnings.warn("Package `nrefocus` not available. Proceeding "
                      "without autofocusing.")
        autofocus = False
    else:
        autofocus = True
    
    # use jobmanager if available
    try:
        import jobmanager as jm
        jm.decorators.decorate_module_ProgressBar(odt, 
                            decorator=jm.decorators.ProgressBarOverrideCount,
                            interval=.1)
    except:
        pass
    

    datazip = get_file("mie_3d_sphere_field.zip")
    # Get simulation data
    arc = zipfile.ZipFile(datazip)

    Ex_real = np.loadtxt(arc.open("mie_sphere_real.txt"))
    Ex_imag = np.loadtxt(arc.open("mie_sphere_imag.txt"))
    Ex = Ex_real +1j*Ex_imag
    # get nm, lD, res
    with arc.open("mie_info.txt") as info:
        cfg = {}
        for l in info.readlines():
            l = l.decode()
            if l.count("=") == 1:
                key, val = l.split("=")
                cfg[key.strip()] = float(val.strip())

    # Manually set number of angles:
    A = 200
    
    print("Example: Backpropagation from 3d Mie scattering")
    print("Refractive index of medium:", cfg["nm"])
    print("Measurement position from object center:", cfg["lD"])
    print("Wavelength sampling:", cfg["res"])
    print("Number of angles for reconstruction:", A)
    print("Perform autofocusing:", autofocus)
    print("Performing backpropagation.")


    # Reconstruction angles
    angles = np.linspace(0,2*np.pi,A,endpoint=False)
    
    if autofocus:
        Ex, d = nrefocus.autofocus(Ex, 
                                   cfg["nm"], 
                                   cfg["res"],
                                   (-1.5*cfg["lD"]*cfg["res"],0),
                                   ret_d=True)
        lD = 0
        print("Autofocusing distance [px]: ", d)
    else:
        lD = cfg["lD"]*cfg["res"]

    # Create sinogram
    u_sin = np.tile(Ex.flat,A).reshape(A, int(cfg["size"]), int(cfg["size"]))

    # Apply the Rytov approximation
    u_sinR = odt.sinogram_as_rytov(u_sin)

    # Backpropagation
    fR = odt.backpropagate_3d(uSin = u_sinR, 
                              angles = angles, 
                              res = cfg["res"],
                              nm = cfg["nm"],
                              lD = lD,
                              padfac=2.1)

    # RI computation
    nR = odt.odt_to_ri(fR, cfg["res"], cfg["nm"])

    # Plotting
    fig, axes = plt.subplots(2, 3, figsize=(12,8), dpi=300)
    axes = np.array(axes).flatten()
    # field
    axes[0].set_title("Mie field phase")
    axes[0].set_xlabel("xD")
    axes[0].set_ylabel("yD")
    axes[0].imshow(np.angle(Ex), cmap=plt.cm.coolwarm)  # @UndefinedVariable
    axes[1].set_title("Mie field amplitude")
    axes[1].set_xlabel("xD")
    axes[1].set_ylabel("yD")
    axes[1].imshow(np.abs(Ex), cmap=plt.cm.gray)  # @UndefinedVariable

    # line plot
    axes[2].set_title("RI reconstruction, line plots")
    axes[2].set_xlabel("distance [px]")
    axes[2].set_ylabel("real refractive index")
    center = int(cfg["size"]/2)
    x=np.arange(cfg["size"]) - center
    axes[2].plot(x, nR[:,center,center].real, label="along x")
    axes[2].plot(x, nR[center,center,:].real, label="along z")
    axes[2].plot(x, nR[center,:,center].real, label="along y")
    axes[2].legend(loc=2)
    axes[2].set_xlim((-center, center))
    dn = abs(cfg["nsph"] - cfg["nm"])
    axes[2].set_ylim((cfg["nm"]-dn/10, cfg["nsph"]+dn))
    axes[2].ticklabel_format(useOffset=False)

    # cross sections
    axes[3].set_title("RI reconstruction, section at x=0")
    axes[3].set_xlabel("z")
    axes[3].set_ylabel("y")
    axes[3].imshow(nR[center,:,:].real)

    axes[4].set_title("RI reconstruction, section at y=0")
    axes[4].set_xlabel("x")
    axes[4].set_ylabel("z")
    axes[4].imshow(nR[:,center,:].real)

    axes[5].set_title("RI reconstruction, section at z=0")
    axes[5].set_xlabel("y")
    axes[5].set_ylabel("x")
    axes[5].imshow(nR[:,:,center].real)

    plt.tight_layout()

    outname = join(DIR, "backprop_from_mie_3d_sphere.png")

    print("Creating output file:", outname)
    plt.savefig(outname)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" 2D reconstruction in optical tomography with the Born approximation

The first Born approximation for a 2D scattering problem with a plane
wave 
:math:`u_0(\mathbf{r}) = a_0 \exp(-ik_\mathrm{m}\mathbf{s_0r})`
reads:

.. math::
    u_\mathrm{B}(\mathbf{r}) = \iint \!\! d^2r' 
        G(\mathbf{r-r'}) f(\mathbf{r'}) u_0(\mathbf{r'})

The Green's function in 2D is the zero-order Hankel function
of the first kind:

.. math::
    G(\mathbf{r-r'}) = \\frac{i}{4} 
        H_0^\mathrm{(1)}(k_\mathrm{m} \\left| \mathbf{r-r'} \\right|) 

Solving for :math:`f(\mathbf{r})` yields the Fourier diffraction theorem
in 2D

.. math::
    \widehat{F}(k_\mathrm{m}(\mathbf{s-s_0})) = 
        - \sqrt{\\frac{2}{\pi}} 
        \\frac{i k_\mathrm{m}}{a_0} M
        \widehat{U}_{\mathrm{B},\phi_0}(k_\mathrm{Dx})
        \exp \! \\left(-i k_\mathrm{m} M l_\mathrm{D} \\right)
    
where 
:math:`\widehat{F}(k_\mathrm{x}, k_\mathrm{z})`
is the Fourier transformed object function and 
:math:`\widehat{U}_{\mathrm{B}, \phi_0}(k_\mathrm{Dx})` is the
Fourier transformed complex wave that travels along :math:`\mathbf{s_0}`
(in the direction of :math:`\phi_0`) measured at the detector
:math:`\mathbf{r_D}`.


The following identities are used:

.. math::
    k_\mathrm{m} (\mathbf{s-s_0}) &= k_\mathrm{Dx} \, \mathbf{t_\perp} +
    k_\mathrm{m}(M - 1) \, \mathbf{s_0}
    
    \mathbf{s_0} &= \\left(p_0 , \, M_0 \\right) = 
    (-\sin\phi_0, \, \cos\phi_0)
    
    \mathbf{t_\\perp} &= \\left(- M_0 , \, p_0 \\right) = 
    (\cos\phi_0, \, \sin\phi_0)


"""
from __future__ import division, print_function

import numpy as np
import scipy.interpolate as intp
import scipy.ndimage

from . import util

__all__ = ["backpropagate_2d", "fourier_map_2d", "sum_2d"]
_verbose = 1


def backpropagate_2d(uSin, angles, res, nm, lD=0, coords=None,
                     weight_angles=True,
                     onlyreal=False, padding=True, padval=0,
                     jmc=None, jmm=None, verbose=_verbose):
    u""" 2D backpropagation with the Fourier diffraction theorem

    Two-dimensional diffraction tomography reconstruction
    algorithm for scattering of a plane wave
    :math:`u_0(\mathbf{r}) = u_0(x,z)` 
    by a dielectric object with refractive index
    :math:`n(x,z)`.

    This method implements the 2D backpropagation algorithm [1]_

    .. math::
        f(\mathbf{r}) = 
            -\\frac{i k_\mathrm{m}}{2\pi}
            \\sum_{j=1}^{N} \! \Delta \phi_0 D_{-\phi_j} \!\!
            \\left \{
            \\text{FFT}^{-1}_{\mathrm{1D}}
            \\left \{
            \\left| k_\mathrm{Dx} \\right|  
            \\frac{\\text{FFT}_{\mathrm{1D}} \\left \{
            u_{\mathrm{B},\phi_j}(x_\mathrm{D}) \\right \}
            }{u_0(l_\mathrm{D})}
            \exp \! \\left[i k_\mathrm{m}(M - 1) \cdot 
            (z_{\phi_j}-l_\mathrm{D}) \\right]
            \\right \} 
            \\right \}

    with the forward :math:`\\text{FFT}_{\mathrm{1D}}` and inverse 
    :math:`\\text{FFT}^{-1}_{\mathrm{1D}}` 1D fast Fourier transform, the
    rotational operator :math:`D_{-\phi_j}`, the angular distance between the
    projections :math:`\Delta \phi_0`, the ramp filter in Fourier space
    :math:`|k_\mathrm{Dx}|`, and the propagation distance 
    :math:`(z_{\phi_j}-l_\mathrm{D})`.

    Parameters
    ----------
    uSin : (A,N) ndarray
        Two-dimensional sinogram of line recordings
        :math:`u_{\mathrm{B}, \phi_j}(x_\mathrm{D})`
        divided by the incident plane wave :math:`u_0(l_\mathrm{D})`
        measured at the detector.
    angles : (A,) ndarray
        Angular positions :math:`\phi_j` of `uSin` in radians.
    res : float
        Vacuum wavelength of the light :math:`\lambda` in pixels.
    nm : float
        Refractive index of the surrounding medium :math:`n_\mathrm{m}`.
    lD : float
        Distance from center of rotation to detector plane 
        :math:`l_\mathrm{D}` in pixels.
    coords : None [(2,M) ndarray]
        Computes only the output image at these coordinates. This
        keyword is reserved for future versions and is not
        implemented yet.
    weight_angles : bool
        If `True`, weights each backpropagated projection with a factor
        proportional to the angular distance between the neighboring
        projections. 
        
        .. math::
            \Delta \phi_0 \\longmapsto \Delta \phi_j = \\frac{\phi_{j+1} - \phi_{j-1}}{2}
        
        .. versionadded:: 0.1.1
    onlyreal : bool
        If `True`, only the real part of the reconstructed image
        will be returned. This saves computation time.
    padding : bool
        Pad the input data to the second next power of 2 before
        Fourier transforming. This reduces artifacts and speeds up
        the process for input image sizes that are not powers of 2.
    padval : float
        The value used for padding. This is important for the Rytov
        approximation, where an approximate zero in the phase might
        translate to 2πi due to the unwrapping algorithm. In that
        case, this value should be a multiple of 2πi. 
        If `padval` is `None`, then the edge values are used for
        padding (see documentation of :func:`numpy.pad`).
    jmc, jmm : instance of :func:`multiprocessing.Value` or `None`
        The progress of this function can be monitored with the 
        :mod:`jobmanager` package. The current step `jmc.value` is
        incremented `jmm.value` times. `jmm.value` is set at the 
        beginning.
    verbose : int
        Increment to increase verbosity.


    Returns
    -------
    f : ndarray of shape (N,N), complex if `onlyreal` is `False`
        Reconstructed object function :math:`f(\mathbf{r})` as defined
        by the Helmholtz equation.
        :math:`f(x,z) = 
        k_m^2 \\left(\\left(\\frac{n(x,z)}{n_m}\\right)^2 -1\\right)`


    See Also
    --------
    odt_to_ri : conversion of the object function :math:`f(\mathbf{r})` 
        to refractive index :math:`n(\mathbf{r})`.

    radontea.backproject : backprojection based on the Fourier slice
        theorem.
    
    Notes
    -----
    Do not use the parameter `lD` in combination with the Rytov
    approximation - the propagation is not correctly described.
    Instead, numerically refocus the sinogram prior to converting
    it to Rytov data (using e.g. :func:`odtbrain.sinogram_as_rytov`)
    with a numerical focusing algorithm (available in the Python
    package :py:mod:`nrefocus`).
    """
    ##
    ##
    # TODO:
    # - combine the 2nd filter and the rotation in the for loop
    # to save memory. However, memory is not a big issue in 2D.
    ##
    ##
    A = angles.shape[0]
    if jmm is not None:
        jmm.value = A + 2
    # Check input data
    assert len(uSin.shape) == 2, "Input data `uB` must have shape (A,N)!"
    assert len(uSin) == A, "`len(angles)` must be  equal to `len(uSin)`!"
    
    if coords is not None:
        raise NotImplementedError("Output coordinates cannot yet" +
                                  " be set for the 2D backrpopagation algorithm.")
    # Cut-Off frequency
    # km [1/px]
    km = (2 * np.pi * nm) / res
    # Here, the notation defines 
    # a wave propagating to the right as:
    #
    #    u0(x) = exp(ikx)
    #
    # However, in physics usually we use the other sign convention:
    #
    #    u0(x) = exp(-ikx)
    #
    # In order to be consistent with programs like Meep or our
    # scattering script for a dielectric cylinder, we want to use the
    # latter sign convention.
    # This is not a big problem. We only need to multiply the imaginary
    # part of the scattered wave by -1.

    # Perform weighting
    if weight_angles:
        weights = util.compute_angle_weights_1d(angles).reshape(-1,1)
        sinogram = uSin * weights
    else:
        sinogram = uSin

    # Size of the input data
    ln = sinogram.shape[1]

    # We perform padding before performing the Fourier transform.
    # This gets rid of artifacts due to false periodicity and also
    # speeds up Fourier transforms of the input image size is not
    # a power of 2.
    order = max(64., 2**np.ceil(np.log(ln * 2.1) / np.log(2)))

    if padding:
        pad = order - ln
    else:
        pad = 0

    padl = np.int(np.ceil(pad / 2))
    padr = np.int(pad - padl)

    if padval is None:
        sino = np.pad(sinogram, ((0, 0), (padl, padr)),
                      mode="edge")
        if verbose > 0:
            print("......Padding with edge values.")
    else:
        sino = np.pad(sinogram, ((0, 0), (padl, padr)),
                      mode="linear_ramp",
                      end_values=(padval,))
        if verbose > 0:
            print("......Verifying padding value: {}".format(padval))

    # zero-padded length of sinogram.
    lN = sino.shape[1]

    # Ask for the filter. Do not include zero (first element).
    #
    # Integrals over ϕ₀ [0,2π]; kx [-kₘ,kₘ]
    #   - double coverage factor 1/2 already included
    #   - unitary angular frequency to unitary ordinary frequency
    #     conversion performed in calculation of UB=FT(uB).
    #
    # f(r) = -i kₘ / ((2π)^(3/2) a₀)            (prefactor)
    #      * iint dϕ₀ dkx                       (prefactor)
    #      * |kx|                               (prefactor)
    #      * exp(-i kₘ M lD )                   (prefactor)
    #      * UBϕ₀(kx)                             (dependent on ϕ₀)
    #      * exp( i (kx t⊥ + kₘ (M - 1) s₀) r )   (dependent on ϕ₀ and r)
    #
    # (r and s₀ are vectors. In the last term we perform the dot-product)
    #
    # kₘM = sqrt( kₘ² - kx² )
    # t⊥  = (  cos(ϕ₀), sin(ϕ₀) )
    # s₀  = ( -sin(ϕ₀), cos(ϕ₀) )
    #
    # The filter can be split into two parts
    #
    # 1) part without dependence on the z-coordinate
    #
    #        -i kₘ / ((2π)^(3/2) a₀)
    #      * iint dϕ₀ dkx
    #      * |kx|
    #      * exp(-i kₘ M lD )
    #
    # 2) part with dependence of the z-coordinate
    #
    #        exp( i (kx t⊥ + kₘ (M - 1) s₀) r )
    #
    # The filter (1) can be performed using the classical filter process
    # as in the backprojection algorithm.
    #
    #
    if jmc is not None:
        jmc.value += 1

    # Corresponding sample frequencies
    fx = np.fft.fftfreq(lN)  # 1D array
    # kx is a 1D array.
    kx = 2 * np.pi * fx
    # Differentials for integral
    dphi0 = 2 * np.pi / A
    # We will later multiply with phi0.
    #               a, x
    kx = kx.reshape(1, -1)
    # Low-pass filter:
    # less-than-or-equal would give us zero division error.
    filter_klp = (kx**2 < km**2)

    # Filter M so there are no nans from the root
    M = 1. / km * np.sqrt((km**2 - kx**2) * filter_klp)

    prefactor = -1j * km / (2 * np.pi)
    prefactor *= dphi0
    prefactor *= np.abs(kx) * filter_klp
    # new in version 0.1.4:
    # We multiply by the factor (M-1) instead of just (M)
    # to take into account that we have a scattered
    # wave that is normalized by u0.
    prefactor *= np.exp(-1j * km * (M-1) * lD)
    # Perform filtering of the sinogram
    projection = np.fft.fft(sino, axis=-1) * prefactor

    #
    # filter (2) must be applied before rotation as well
    # exp( i (kx t⊥ + kₘ (M - 1) s₀) r )
    #
    # t⊥  = (  cos(ϕ₀), sin(ϕ₀) )
    # s₀  = ( -sin(ϕ₀), cos(ϕ₀) )
    #
    # This filter is effectively an inverse Fourier transform
    #
    # exp(i kx xD) exp(i kₘ (M - 1) yD )
    #
    # xD =   x cos(ϕ₀) + y sin(ϕ₀)
    # yD = - x sin(ϕ₀) + y cos(ϕ₀)

    # Everything is in pixels
    center = ln / 2.0
    x = np.arange(lN) - center + .5
    # Meshgrid for output array
    yv = x.reshape(-1, 1)

    Mp = M.reshape(1, -1)
    filter2 = np.exp(1j * yv * km * (Mp - 1))  # .reshape(1,lN,lN)

    projection = projection.reshape(A, 1, lN)  # * filter2


    # Prepare complex output image
    if onlyreal:
        outarr = np.zeros((ln, ln))
    else:
        outarr = np.zeros((ln, ln), dtype=np.dtype(complex))

    if jmc is not None:
        jmc.value += 1

    # Calculate backpropagations
    for i in np.arange(A):
        # Create an interpolation object of the projection.

        # interpolation of the rotated fourier transformed projection
        # this is already tiled onto the entire image.
        sino_filtered = np.fft.ifft(projection[i] * filter2, axis=-1)

        # Resize filtered sinogram back to original size
        sino = sino_filtered[:ln, padl:padl + ln]

        rotated_projr = scipy.ndimage.interpolation.rotate(
            sino.real, -angles[i] * 180 / np.pi,
            reshape=False, mode="constant", cval=0)
        # Append results

        outarr += rotated_projr

        if not onlyreal:
            outarr += 1j * scipy.ndimage.interpolation.rotate(
                sino.imag, -angles[i] * 180 / np.pi,
                reshape=False, mode="constant", cval=0)

        if jmc is not None:
            jmc.value += 1

    return outarr


def fourier_map_2d(uSin, angles, res, nm, lD=0, semi_coverage=False,
                   coords=None, jmc=None, jmm=None, verbose=_verbose):
    u""" 2D Fourier mapping with the Fourier diffraction theorem

    Two-dimensional diffraction tomography reconstruction
    algorithm for scattering of a plane wave
    :math:`u_0(\mathbf{r}) = u_0(x,z)` 
    by a dielectric object with refractive index
    :math:`n(x,z)`.

    This function implements the solution by interpolation in
    Fourier space.

    Parameters
    ----------
    uSin : (A,N) ndarray
        Two-dimensional sinogram of line recordings
        :math:`u_{\mathrm{B}, \phi_j}(x_\mathrm{D})`
        divided by the incident plane wave :math:`u_0(l_\mathrm{D})`
        measured at the detector.
    angles : (A,) ndarray
        Angular positions :math:`\phi_j` of `uSin` in radians.
    res : float
        Vacuum wavelength of the light :math:`\lambda` in pixels.
    nm : float
        Refractive index of the surrounding medium :math:`n_\mathrm{m}`.
    lD : float
        Distance from center of rotation to detector plane 
        :math:`l_\mathrm{D}` in pixels.
    semi_coverage : bool
        If set to `True`, it is assumed that the sinogram does not 
        necessarily cover the full angular range from 0 to 2π, but an
        equidistant coverage over 2π can be achieved by inferring point
        (anti)symmetry of the (imaginary) real parts of the Fourier 
        transform of f. Valid for any set of angles {X} that result in
        a 2π coverage with the union set {X}U{X+π}.
    coords : None [(2,M) ndarray]
        Computes only the output image at these coordinates. This
        keyword is reserved for future versions and is not
        implemented yet.
    jmc, jmm : instance of :func:`multiprocessing.Value` or `None`
        The progress of this function can be monitored with the 
        :mod:`jobmanager` package. The current step `jmc.value` is
        incremented `jmm.value` times. `jmm.value` is set at the 
        beginning.
    verbose : int
        Increment to increase verbosity.


    Returns
    -------
    f : ndarray of shape (N,N), complex if `onlyreal` is `False`
        Reconstructed object function :math:`f(\mathbf{r})` as defined
        by the Helmholtz equation.
        :math:`f(x,z) = 
        k_m^2 \\left(\\left(\\frac{n(x,z)}{n_m}\\right)^2 -1\\right)`


    See Also
    --------
    backpropagate_2d : implementation by backpropagation
    odt_to_ri : conversion of the object function :math:`f(\mathbf{r})` 
        to refractive index :math:`n(\mathbf{r})`.

    Notes
    -----
    Do not use the parameter `lD` in combination with the Rytov
    approximation - the propagation is not correctly described.
    Instead, numerically refocus the sinogram prior to converting
    it to Rytov data (using e.g. :func:`odtbrain.sinogram_as_rytov`)
    with a numerical focusing algorithm (available in the Python
    package :py:mod:`nrefocus`).
    """
    ##
    ##
    # TODO:
    # - zero-padding as for backpropagate_2D - However this is not
    # necessary as Fourier interpolation is not parallelizable with
    # multiprocessing and thus unattractive. Could be interesting for
    # specific environments without the Python GIL.
    # - Deal with oversampled data. Maybe issue a warning.
    ##
    ##
    A = angles.shape[0]
    if jmm is not None:
        jmm.value = 4
    # Check input data
    assert len(uSin.shape) == 2, "Input data `uSin` must have shape (A,N)!"
    assert len(uSin) == A, "`len(angles)` must be  equal to `len(uSin)`!"
    
    if coords is not None:
        raise NotImplementedError("Output coordinates cannot yet" +
                                  " be set for the 2D backrpopagation algorithm.")
    # Cut-Off frequency
    # km [1/px]
    km = (2 * np.pi * nm) / res

    # Fourier transform of all uB's
    # In the script we used the unitary angular frequency (uaf) Fourier
    # Transform. The discrete Fourier transform is equivalent to the
    # unitary ordinary frequency (uof) Fourier transform.
    #
    # uof: f₁(ξ) = int f(x) exp(-2πi xξ)
    #
    # uaf: f₃(ω) = (2π)^(-n/2) int f(x) exp(-i ωx)
    #
    # f₁(ω/(2π)) = (2π)^(n/2) f₃(ω)
    # ω = 2πξ
    #
    # Our Backpropagation Formula is with uaf convention of the Form
    #
    # F(k) = 1/sqrt(2π) U(kD)
    #
    # If we convert now to uof convention, we get
    #
    # F(k) = U(kD)
    #
    # This means that if we divide the Fourier transform of the input
    # data by sqrt(2π) to convert f₃(ω) to f₁(ω/(2π)), the resulting
    # value for F is off by a factor of 2π.
    #
    # Instead, we can just multiply *UB* by sqrt(2π) and calculate
    # everything in uof.
    #UB =  np.fft.fft(np.fft.ifftshift(uSin, axes=-1))/np.sqrt(2*np.pi)
    #
    #
    # Furthermore, we define 
    # a wave propagating to the right as:
    #
    #    u0(x) = exp(ikx)
    #
    # However, in physics usually we use the other sign convention:
    #
    #    u0(x) = exp(-ikx)
    #
    # In order to be consistent with programs like Meep or our
    # scattering script for a dielectric cylinder, we want to use the
    # latter sign convention.
    # This is not a big problem. We only need to multiply the imaginary
    # part of the scattered wave by -1.

    # The radon transform introduced a shift in the data, which is not
    # conform with the FFT algorithm.
    UB = np.fft.fft(np.fft.ifftshift(uSin, axes=-1)) * np.sqrt(2 * np.pi)

    # Corresponding sample frequencies
    fx = np.fft.fftfreq(len(uSin[0]))  # 1D array

    # kx is an 1D array.
    kx = 2 * np.pi * fx

    if jmc is not None:
        jmc.value += 1

    # Undersampling/oversampling?
    # Determine if the resolution of the image is too low by looking
    # at the maximum value for kx. This is no comparison between
    # Nyquist and Rayleigh frequency.
    if verbose and np.max(kx**2) <= km**2:
        # Detector is not set up properly. Higher resolution
        # can be achieved.
        print("......Measurement data is undersampled.")
    else:
        print("......Measurement data is oversampled.")
        # raise NotImplementedError("Oversampled data not yet supported."+
        #                   " Please rescale xD-axis of the input data.")
        # DEAL WITH OVERSAMPLED DATA?
        #import IPython
        # IPython.embed()
        #lenk = len(kx)
        #kx = np.fft.ifftshift(np.linspace(-np.sqrt(km),np.sqrt(km),len(fx), endpoint=False))

    #
    # F(kD-kₘs₀) = - i kₘ sqrt(2/π) / a₀ * M exp(-i kₘ M lD) * UB(kD)
    # kₘM = sqrt( kₘ² - kx² )
    # s₀  = ( -sin(ϕ₀), cos(ϕ₀) )
    #
    # We create the 2D interpolation object F
    #   - We compute the real coordinates (krx,kry) = kD-kₘs₀
    #   - We set as grid points the right side of the equation
    #
    # The interpolated griddata may go up to sqrt(2)*kₘ for kx and ky.

    kx = kx.reshape(1, -1)
    # a0 should have same shape as kx and UB
    #a0 = np.atleast_1d(a0)
    #a0 = a0.reshape(1,-1)

    filter_klp = (kx**2 < km**2)
    M = 1. / km * np.sqrt(km**2 - kx**2)
    #Fsin =  -1j * km * np.sqrt(2/np.pi) / a0 * M * np.exp(-1j*km*M*lD)
    # new in version 0.1.4:
    # We multiply by the factor (M-1) instead of just (M)
    # to take into account that we have a scattered
    # wave that is normalized by u0.
    Fsin = -1j * km * np.sqrt(2 / np.pi) * M * np.exp(-1j * km * (M-1) * lD)

    # UB has same shape (len(angles), len(kx))
    Fsin = Fsin * UB * filter_klp

    ang = angles.reshape(-1, 1)

    if semi_coverage:
        Fsin = np.vstack((Fsin, np.conj(Fsin)))
        ang = np.vstack((ang, ang + np.pi))


    if jmc is not None:
        jmc.value += 1

    # Compute kxl and kyl (in rotated system ϕ₀)
    kxl = kx
    kyl = np.sqrt((km**2 - kx**2) * filter_klp) - km
    # rotate kxl and kyl to where they belong
    krx = np.cos(ang) * kxl + np.sin(ang) * kyl
    kry = - np.sin(ang) * kxl + np.cos(ang) * kyl

    Xf = krx.flatten()
    Yf = kry.flatten()
    Zf = Fsin.flatten()

    # DEBUG: plot kry vs krx
    #from matplotlib import pylab as plt
    # plt.figure()
    # for i in range(len(krx)):
    #    plt.plot(krx[i],kry[i],"x")
    # plt.axes().set_aspect('equal')
    # plt.show()

    # interpolation on grid with same resolution as input data
    kintp = np.fft.fftshift(kx.reshape(-1))

    Fcomp = intp.griddata((Xf, Yf), Zf, (kintp[None, :], kintp[:, None]))

    if jmc is not None:
        jmc.value += 1

    # removed nans
    Fcomp[np.where(np.isnan(Fcomp))] = 0

    # Filter data
    kinx, kiny = np.meshgrid(np.fft.fftshift(kx), np.fft.fftshift(kx))
    Fcomp[np.where((kinx**2 + kiny**2) > np.sqrt(2) * km)] = 0

    # DEBUG: Output filtered fourier image
    #proc_arr2im(Fcomp, scale=True).save("Fourier.bmp")

    #Fcomp[np.where(kinx**2+kiny**2<km)] = 0

    # Fcomp is centered at K = 0 due to the way we chose kintp/coords
    f = np.fft.fftshift(np.fft.ifft2(np.fft.ifftshift(Fcomp)))

    if jmc is not None:
        jmc.value += 1

    return f[::-1]


def sum_2d(uSin, angles, res, nm, lD=0, coords=None,
           jmc=None, jmm=None, verbose=_verbose):
    u""" 2D sum-reconstruction with the Fourier diffraction theorem

    Two-dimensional diffraction tomography reconstruction
    algorithm for scattering of a plane wave
    :math:`u_0(\mathbf{r}) = u_0(x,z)` 
    by a dielectric object with refractive index
    :math:`n(x,z)`.

    This function implements the solution by summation in real
    space, which is extremely slow.

    Parameters
    ----------
    uSin : (A,N) ndarray
        Two-dimensional sinogram of line recordings
        :math:`u_{\mathrm{B}, \phi_j}(x_\mathrm{D})`
        divided by the incident plane wave :math:`u_0(l_\mathrm{D})`
        measured at the detector.
    angles : (A,) ndarray
        Angular positions :math:`\phi_j` of `uSin` in radians.
    res : float
        Vacuum wavelength of the light :math:`\lambda` in pixels.
    nm : float
        Refractive index of the surrounding medium :math:`n_\mathrm{m}`.
    lD : float
        Distance from center of rotation to detector plane 
        :math:`l_\mathrm{D}` in pixels.
    coords : None or (2,M) ndarray]
        Computes only the output image at these coordinates. This
        keyword is reserved for future versions and is not
        implemented yet.
    jmc, jmm : instance of :func:`multiprocessing.Value` or `None`
        The progress of this function can be monitored with the 
        :mod:`jobmanager` package. The current step `jmc.value` is
        incremented `jmm.value` times. `jmm.value` is set at the 
        beginning.
    verbose : int
        Increment to increase verbosity.

    Returns
    -------
    f : ndarray of shape (N,N), complex if `onlyreal` is `False`
        Reconstructed object function :math:`f(\mathbf{r})` as defined
        by the Helmholtz equation.
        :math:`f(x,z) = 
        k_m^2 \\left(\\left(\\frac{n(x,z)}{n_m}\\right)^2 -1\\right)`


    See Also
    --------
    backpropagate_2d : implementation by backprojection
    fourier_map_2d : implementation by Fourier interpolation
    odt_to_ri : conversion of the object function :math:`f(\mathbf{r})` 
        to refractive index :math:`n(\mathbf{r})`.


    Notes
    -----
    This method is not meant for production use. The computation time
    is very long and the reconstruction quality is bad. This function
    is included in the package, because of its educational value,
    exemplifying the backpropagation algorithm.

    Do not use the parameter `lD` in combination with the Rytov
    approximation - the propagation is not correctly described.
    Instead, numerically refocus the sinogram prior to converting
    it to Rytov data (using e.g. :func:`odtbrain.sinogram_as_rytov`)
    with a numerical focusing algorithm (available in the Python
    package :py:mod:`nrefocus`).
    """
    if coords is None:
        lx = uSin.shape[1]
        x = np.linspace(-lx/2, lx/2, lx, endpoint=False)
        xv, yv = np.meshgrid(x,x)
        coords = np.zeros((2, lx**2))
        coords[0,:] = xv.flat
        coords[1,:] = yv.flat
    
    
    if jmm is not None:
        jmm.value = coords.shape[1] + 1
    # Cut-Off frequency
    km = (2 * np.pi * nm) / res

    # Fourier transform of all uB's
    # In the script we used the unitary angular frequency (uaf) Fourier
    # Transform. The discrete Fourier transform is equivalent to the
    # unitary ordinary frequency (uof) Fourier transform.
    #
    # uof: f₁(ξ) = int f(x) exp(-2πi xξ)
    #
    # uaf: f₃(ω) = (2π)^(-n/2) int f(x) exp(-i ωx)
    #
    # f₁(ω/(2π)) = (2π)^(n/2) f₃(ω)
    # ω = 2πξ
    #
    # We have a one-dimensional (n=1) Fourier transform and UB in the
    # script is equivalent to f₃(ω). Because we are working with the
    # uaf, we divide by sqrt(2π) after computing the fft with the uof.
    #
    # We calculate the fourier transform of uB further below. This is
    # necessary for memory control.

    # Corresponding sample frequencies
    fx = np.fft.fftfreq(uSin[0].shape[0])  # 1D array
    # kx is a 1D array.
    kx = 2 * np.pi * fx

    # Undersampling/oversampling?
    # Determine if the resolution of the image is too low by looking
    # at the maximum value for kx. This is no comparison between
    # Nyquist and Rayleigh frequency.
    if np.max(kx**2) <= 2 * km**2:
        # Detector is not set up properly. Higher resolution
        # can be achieved.
        if verbose:
            print("......Measurement data is undersampled.")
    else:
        if verbose:
            print("......Measurement data is oversampled.")
        raise NotImplementedError("Oversampled data not yet supported." +
                                  " Please rescale input data")

    # Differentials for integral
    dphi0 = 2 * np.pi / len(angles)
    dkx = kx[1] - kx[0]

    # We will later multiply with phi0.
    # Make sure we are using correct shapes
    kx = kx.reshape(1, kx.shape[0])

    # Low-pass filter:
    # less-than-or-equal would give us zero division error.
    filter_klp = (kx**2 < km**2)

    # a0 will be multiplied with kx
    #a0 = np.atleast_1d(a0)
    #a0 = a0.reshape(1,-1)

    # Create the integrand
    # Integrals over ϕ₀ [0,2π]; kx [-kₘ,kₘ]
    #   - double coverage factor 1/2 already included
    #   - unitary angular frequency to unitary ordinary frequency
    #     conversion performed in calculation of UB=FT(uB).
    #
    # f(r) = -i kₘ / ((2π)^(3/2) a₀)            (prefactor)
    #      * iint dϕ₀ dkx                       (prefactor)
    #      * |kx|                               (prefactor)
    #      * exp(-i kₘ M lD )                   (prefactor)
    #      * UBϕ₀(kx)                             (dependent on ϕ₀)
    #      * exp( i (kx t⊥ + kₘ (M - 1) s₀) r )   (dependent on ϕ₀ and r)
    #
    # (r and s₀ are vectors. In the last term we perform the dot-product)
    #
    # kₘM = sqrt( kₘ² - kx² )
    # t⊥  = (  cos(ϕ₀), sin(ϕ₀) )
    # s₀  = ( -sin(ϕ₀), cos(ϕ₀) )
    #
    #
    # everything that is not dependent on phi0:
    #
    # Filter M so there are no nans from the root
    M = 1. / km * np.sqrt((km**2 - kx**2) * filter_klp)
    prefactor = -1j * km / ((2 * np.pi)**(3. / 2))
    prefactor *= dphi0 * dkx
    # Also filter the prefactor, so nothing outside the required
    # low-pass contributes to the sum.
    prefactor *= np.abs(kx) * filter_klp
    # new in version 0.1.4:
    # We multiply by the factor (M-1) instead of just (M)
    # to take into account that we have a scattered
    # wave that is normalized by u0.
    prefactor *= np.exp(-1j * km * (M-1) * lD)

    # Initiate function f
    f = np.zeros(len(coords[0]), dtype=np.complex128)
    lenf = len(f)
    lenu0 = len(uSin[0])  # lenu0 = len(kx[0])

    # Initiate vector r that corresponds to calculating a value of f.
    r = np.zeros((2, 1, 1))

    # Everything is normal.
    # Get the angles ϕ₀.
    phi0 = angles.reshape(-1, 1)
    # Compute the Fourier transform of uB.
    # This is true: np.fft.fft(UB)[0] == np.fft.fft(UB[0])
    # because axis -1 is always used.
    #
    #
    # Furthermore, The notation in the our optical tomography script for
    # a wave propagating to the right is:
    #
    #    u0(x) = exp(ikx)
    #
    # However, in physics usually usethe other sign convention:
    #
    #    u0(x) = exp(-ikx)
    #
    # In order to be consisten with programs like Meep or our scattering
    # script for a dielectric cylinder, we want to use the latter sign
    # convention.
    # This is not a big problem. We only need to multiply the imaginary
    # part of the scattered wave by -1.
    UB = np.fft.fft(np.fft.ifftshift(uSin, axes=-1)) / np.sqrt(2 * np.pi)
    UBi = UB.reshape(len(angles), lenu0)

    if jmc is not None:
        jmc.value += 1

    for j in range(lenf):
        # Get r (We compute f(r) in this for-loop)
        r[0][:] = coords[0, j]  # x
        r[1][:] = coords[1, j]  # y

        # Integrand changes with r, so we have to create a new
        # array:
        integrand = prefactor * UBi

        # We save memory by directly applying the following to
        # the integrand:
        #
        # Vector along which we measured
        #s0 = np.zeros((2, phi0.shape[0], kx.shape[0]))
        #s0[0] = -np.sin(phi0)
        #s0[1] = +np.cos(phi0)

        # Vector perpendicular to s0
        #t_perp_kx = np.zeros((2, phi0.shape[0], kx.shape[1]))
        #
        #t_perp_kx[0] = kx*np.cos(phi0)
        #t_perp_kx[1] = kx*np.sin(phi0)

        #
        #term3 = np.exp(1j*np.sum(r*( t_perp_kx + (gamma-km)*s0 ), axis=0))
        # integrand* = term3
        #
        # Reminder:
        # f(r) = -i kₘ / ((2π)^(3/2) a₀)            (prefactor)
        #      * iint dϕ₀ dkx                       (prefactor)
        #      * |kx|                               (prefactor)
        #      * exp(-i kₘ M lD )                   (prefactor)
        #      * UB(kx)                             (dependent on ϕ₀)
        #      * exp( i (kx t⊥ + kₘ(M - 1) s₀) r )   (dependent on ϕ₀ and r)
        #
        # (r and s₀ are vectors. In the last term we perform the dot-product)
        #
        # kₘM = sqrt( kₘ² - kx² )
        # t⊥  = (  cos(ϕ₀), sin(ϕ₀) )
        # s₀  = ( -sin(ϕ₀), cos(ϕ₀) )
        integrand *= np.exp(1j * (
            r[0] * (kx * np.cos(phi0) - km * (M - 1) * np.sin(phi0)) +
            r[1] * (kx * np.sin(phi0) + km * (M - 1) * np.cos(phi0))))

        # Calculate the integral for the position r
        # integrand.sort()
        f[j] = np.sum(integrand)

        # free memory
        del integrand

        if jmc is not None:
            jmc.value += 1

    return f.reshape(lx,lx)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" Tests fourier_map algorithm
"""
from __future__ import division, print_function

import numpy as np
import os
from os.path import abspath, dirname, join, split
import sys


# Add parent directory to beginning of path variable
DIR = dirname(abspath(__file__))
sys.path = [split(DIR)[0]] + sys.path

import odtbrain._Back_2D

from common_methods import create_test_sino_2d, create_test_sino_3d, cutout, get_test_parameter_set, write_results, get_results

WRITE_RES = False


def test_2d_fmap():
    myframe = sys._getframe()
    myname = myframe.f_code.co_name
    print("running ", myname)
    sino, angles = create_test_sino_2d()
    parameters = get_test_parameter_set(2)
    r = list()
    for p in parameters:
        f = odtbrain._Back_2D.fourier_map_2d(sino, angles, **p)
        r.append(cutout(f))
    if WRITE_RES:
        write_results(myframe, r)
    assert np.allclose(np.array(r).flatten().view(float), get_results(myframe))
    

if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()
